import { assertTypeIsHandled } from '../utils.mjs';
import { mouseIsInArea } from './checks.mjs';

/**
 *
 * @param {import("../engine.mjs").Game} game
 * @param {boolean} is_enabled
 * @param {import("../types").DisplayButtonInformation} button
 * @returns {{
 *  width: number,
 *  height: number,
 *  is_hovered: boolean,
 * }}
 */
export function renderButton(game, is_enabled, button) {
  const current_x = button.x;
  const size = (() => {
    if (button.content.type === 'text') {
      game.context.font = `${button.content.font_size}px sans-serif`;
      const text_size = game.context.measureText(button.content.text);
      return {
        width: text_size.width + 2 * button.content.padding.x,
        height: button.content.font_size * 1.3 + 2 * button.content.padding.y,
      };
    }
    if (button.content.type === 'fixed') {
      return {
        width: button.content.width,
        height: button.content.height,
      };
    }
    if (button.content.type === 'fixed_text') {
      return {
        width: button.content.width,
        height: button.content.height,
      };
    }
    assertTypeIsHandled(button.content, 'button content type');
  })();
  let is_hovered = false;
  let background_color = button.background.base;
  if (mouseIsInArea(game, { x: current_x, y: button.y, width: size.width, height: size.height })) {
    is_hovered = true;
    if (is_enabled) background_color = button.background.hover;
  }
  game.context.fillStyle = background_color;
  game.context.fillRect(button.x, button.y, size.width, size.height);
  if (button.content.type === 'text') {
    game.context.font = `${button.content.font_size}px sans-serif`;
    game.context.fillStyle = 'black';
    game.context.fillText(
      button.content.text,
      button.x + button.content.padding.x,
      button.y + button.content.padding.y + button.content.font_size,
    );
  }
  if (button.content.type === 'fixed_text') {
    game.context.font = `${button.content.font_size}px sans-serif`;
    game.context.fillStyle = 'black';
    const text_size = game.context.measureText(button.content.text);
    if (text_size.width < button.content.width) {
      game.context.fillText(
        button.content.text,
        button.x + (button.content.width - text_size.width) / 2,
        button.y + button.content.font_size + (button.content.height - button.content.font_size * 1.3) / 2,
      );
    } else {
      game.context.fillText(
        button.content.text,
        button.x,
        button.y + button.content.font_size + (button.content.height - button.content.font_size * 1.3) / 2,
        button.content.width,
      );
    }
  }
  return {
    is_hovered: is_hovered,
    width: size.width,
    height: size.height,
  };
}
