import { mouseIsInArea } from './checks.mjs';

/**
 * @param {import("../engine.mjs").Game} game
 * @param {boolean} is_enabled
 * @param {{ x: number, y: number, width: number, height: number, elevator_width: number, border_size: number }} layout
 * @param {{ border: string, background: string, elevator: string, hovered_elevator: string }} color
 * @param {import('../types').ScrollController} controller
 */
export function showHorizontalSlider(game, is_enabled, layout, color, controller) {
  const content_start_x = layout.x;
  const content_start_y = layout.y;
  const content_width = layout.width;
  const content_height = layout.height;
  const view_start_x = layout.x + layout.border_size;
  const view_start_y = layout.y + layout.border_size;
  const view_width = layout.width - 2 * layout.border_size;
  const view_height = layout.height - 2 * layout.border_size;
  const elevator_width = layout.elevator_width;

  // Border and text contents
  game.context.strokeStyle = color.border;
  game.context.strokeRect(layout.x, layout.y, layout.width, layout.height);
  game.context.strokeStyle = 'none';
  game.context.fillStyle = color.background;
  game.context.fillRect(view_start_x, view_start_y, view_width, view_height);

  // Elevator sidebar and contents
  const elevator_start_x = view_start_x + (view_width - elevator_width) * controller.scroll_offset;

  const is_elevator_hovered = mouseIsInArea(game, {
    x: elevator_start_x,
    y: view_start_y,
    width: elevator_width,
    height: view_height,
  });

  game.context.fillStyle = is_enabled && is_elevator_hovered ? color.hovered_elevator : color.elevator;
  game.context.fillRect(elevator_start_x, view_start_y, elevator_width, view_height);
  if (is_enabled && game.mouse.primary_button && is_elevator_hovered) {
    controller.is_elevator_selected = true;
  }

  if (!game.mouse.primary_button) {
    controller.is_elevator_selected = false;
  }

  if (controller.is_elevator_selected) {
    const equivalent_center = game.mouse.x - view_start_x - elevator_width / 2;
    const equivalent_width = view_width - elevator_width;
    controller.scroll_offset = equivalent_center / equivalent_width;
  }

  if (mouseIsInArea(game, { x: content_start_x, y: content_start_y, width: content_width, height: content_height })) {
    if (is_enabled && game.mouse.scroll_down) controller.scroll_offset -= 0.05;
    if (is_enabled && game.mouse.scroll_up) controller.scroll_offset += 0.05;
  }
  if (controller.scroll_offset > 1.0) controller.scroll_offset = 1.0;
  if (controller.scroll_offset < 0.0) controller.scroll_offset = 0.0;
}
