import { assertTypeIsHandled } from '../utils.mjs';
import { getAllLiveTransmutedAndDustedCharacters } from './utils.mjs';

/**
 * @param {import("../types").GameData} game_data
 * @param {import("../types").StoredCharacterData} character
 * @returns {string[]}
 */
export function generateReadableBackstory(game_data, character) {
  const characters = getAllLiveTransmutedAndDustedCharacters(game_data);
  /** @type {Record<import('../types').CharacterId, string>} */
  const names = {};
  for (const character of characters) {
    names[character.id] = character.name;
  }

  /** @type {string[]} */
  const story = [];
  for (const event of character.story.events) {
    if (event.location === 'origins') {
      for (const content of event.content) {
        readableOriginsBackstoryEvent(story, character, event, content, names);
      }
      continue;
    }
    if (event.location === 'town') {
      for (const content of event.content) {
        readableTownBackstoryEvent(story, character, event, content, names);
      }
      continue;
    }
    if (event.location === 'exploration') {
      for (const content of event.content) {
        readableExplorationBackstoryEvent(story, character, event, content, names);
      }
      continue;
    }
    if (event.location === 'captured') {
      for (const content of event.content) {
        readableCapturedBackstoryEvent(story, character, event, content, names);
      }
      continue;
    }
    if (event.location === 'transmuted') {
      for (const content of event.content) {
        readableTransmutedBackstoryEvent(story, character, event, content, names);
      }
      continue;
    }
    assertTypeIsHandled(event, 'backstory event');
  }
  return story;
}

/**
 * @param {Record<import('../types').CharacterId, string>} names
 * @param {import('../types').EventCharacterInfo} info
 */
function getName(names, info) {
  if (info.does_not_recognize_target) return 'someone';
  if (info.relationship.is_partner) {
    return `her girlfriend ${names[info.character]}`;
  } else if (info.relationship.is_friend) {
    return `her friend ${names[info.character]}`;
  } else if (info.relationship.is_rival_bully) {
    return `her rival ${names[info.character]}`;
  } else if (info.relationship.is_rival_target) {
    return `her rival ${names[info.character]}`;
  }
  return names[info.character];
}

/**
 * @param {Record<import('../types').CharacterId, string>} names
 * @param {import('../types').EventCharacterInfo[]} info
 */
function getNames(names, info) {
  let result = '';
  for (let index = 0; index < info.length; ++index) {
    result += getName(names, info[index]);
    if (index === info.length - 1) {
    } else if (index === info.length - 2) {
      result += ' and ';
    } else {
      result += ', ';
    }
  }
  return result;
}

/**
 * @param {import('../types').StoredCharacterData} character
 * @param {Record<import('../types').CharacterId, string>} names
 * @param {import('../types').EventCharacterInfo[]} info
 */
function getCharacterAndNames(character, names, info) {
  let result = character.name;
  for (let index = 0; index < info.length; ++index) {
    if (index === info.length - 1) {
      result += ' and ';
    } else {
      result += ', ';
    }
    result += getName(names, info[index]);
  }
  return result;
}

/**
 * @param {import('../types').EventEnemyInfo[]} enemies
 */
function getEnemies(enemies) {
  return 'a group of mages';
}

/**
 * @param {string[]} story
 * @param {import('../types').StoredCharacterData} character
 * @param {import('../types').CharacterBackstoryEvent} event
 * @param {import('../types').CharacterBackstoryOriginsEvent} content
 * @param {Record<import('../types').CharacterId, string>} names
 */
function readableOriginsBackstoryEvent(story, character, event, content, names) {
  if (content.type === 'vagrant') {
    story.push(
      `Day ${content.cycle}: ${character.name} is an orphan, who grew up raised by the community. She discovers in her teen that she has access to magic, but keeps it hidden for her own safety. As she grows up, she manages to make a living on her own through finding odd jobs, until one day, ${character.name} starts to feel the pull of the Hidden Lands.`,
    );
    return;
  }
  if (content.type === 'vagrant_magic') {
    story.push(
      `Day ${content.cycle}: ${character.name} is an orphan, who grew up raised by the community. As she starts getting older, ${character.name} discovers that she is able to use magic. She tries to keep it hidden from the people that shouldn't know about it, but manages to find a mentor that teaches her the basics of how to use it. She grows up using her skills in secret, but the pull of the Hidden Lands made itself feel as she got old enough.`,
    );
    return;
  }
  if (content.type === 'commoner') {
    story.push(
      `Day ${content.cycle}: ${character.name} is a commoner, from a small family. They aren't especially well-known nor wealthy, but they're doing okay, and ${character.name} grows up like any kid really should, happy and carefree. In her teens, she discovers her own access to magic, but keeps it hidden while trying to keep living a completely average life. However, this all changes when ${character.name} starts to feel the pull of the Hidden Lands.`,
    );
    return;
  }
  if (content.type === 'commoner_money') {
    story.push(
      `Day ${content.cycle}: ${character.name} is a commoner, from a small family. They aren't especially well-known but her family manages to acquire some wealth, which means ${character.name} was destined to be a fairly well-off heiress. The arrival of magic in her life as her teens went by changed that, despite her hiding it and ignoring it for as long as she could. However, as her tweens rolled by, the pull of the Hidden Lands started to rise in her, determining the real direction of her life.`,
    );
    return;
  }
  if (content.type === 'commoner_magic') {
    story.push(
      `Day ${content.cycle}: ${character.name} is a commoner, from a small family. They aren't especially well-known nor wealthy, but as ${character.name} grows up, she discovers that she is able to use magic. Thankfully, her parents help her find a mentor that teaches her the basics of using it, and this means that ${character.name} also learns what the pull of the Hidden Lands is fairly early. She spends some time preparing herself for the journey that might become the next big step in her life, until one day, partly to her relief, she starts feeling it.`,
    );
    return;
  }
  if (content.type === 'noble_money') {
    story.push(
      `Day ${content.cycle}: ${character.name} is the daughter of a fairly well-off aristocratic family. As she grows up, ${character.name} hides her burgeoning magic while she learns about the manners and usages of the courts of the country, and gets trained in diplomacy, science and combat as a true heiress should be. She was destined to lead her family when the times come, however, her magic didn't allow it. The pull of the Hidden Lands caught up with her just as she started what she hoped would become her career, upending her plans in one fell swoop.`,
    );
    return;
  }
  if (content.type === 'noble_magic') {
    story.push(
      `Day ${content.cycle}: ${character.name} is the daughter of a fairly well-off aristocratic family. As she grows up, ${character.name} discovers that she is able to use magic, and when she tells her family, they immediately shift their focus towards her other siblings while she got relegated to learning how to use it with a mentor that her parents hired. She feels the pull of the Hidden Lands soon after she reaches adulthood, and she unexpectedly feels relief when it happens, immediately starting to plan her journey out of her oppressing life and into a better one.`,
    );
    return;
  }
  if (content.type === 'rival_encounter_childhood_bullies') {
    story.push(
      `Day ${content.cycle}: ${character.name} met ${getName(names, content.rival)} while she was a child, the both of them growing up near each other but never really getting along the usual way. ${character.name} had a lot of fun playing pranks on ${getName(names, content.rival)} over the years, ranging from playfights that she usually won, to making ${getName(names, content.rival)} have to figure her way out of tight situations. It made for good memories, but ${getName(names, content.rival)} doesn't seem to always agree about that.`,
    );
    return;
  }
  if (content.type === 'rival_encounter_childhood_is_bullied') {
    story.push(
      `Day ${content.cycle}: ${character.name} met ${getName(names, content.rival)} while she was a child, the both of them growing up near each other but never really getting along the usual way. ${character.name} did get on the bad end of ${getName(names, content.rival)}'s various pranks over the years, ranging from playfights that she usually lost, up to that one time where ${getName(names, content.rival)} tied her up behind a barn and left her to struggle out on her own. She doesn't keep a fond memory of everything that happened, for sure.`,
    );
    return;
  }
  if (content.type === 'rival_encounter_adulthood_bullies') {
    story.push(
      `Day ${content.cycle}: ${character.name} met ${getName(names, content.rival)} while furthering her combat training, and their first encounter ended with ${getName(names, content.rival)} on the ground while ${character.name} stood triumphant over her. As the years passed, ${character.name} often met with ${getName(names, content.rival)} again, and very often found ways to annoy or mess with her, like that one time after another practice that ${getName(names, content.rival)} lost where she tied her hands and took her clothes to keep her fighting harder the next time - good memories! ${character.name} does like how fun ${getName(names, content.rival)} is to play with, though it seems that ${getName(names, content.rival)} doesn't always agree.`,
    );
    return;
  }
  if (content.type === 'rival_encounter_adulthood_is_bullied') {
    story.push(
      `Day ${content.cycle}: ${character.name} met ${getName(names, content.rival)} while furthering her combat training, and their first encounter ended with ${character.name} on the ground while ${getName(names, content.rival)} stood triumphant over her. This seemed to become a staple of ${character.name}'s life, with ${getName(names, content.rival)} always finding an edge to humiliate ${character.name}, and never caring about what she made ${character.name} go through as a result - ${character.name} still remembers having to sneak through the training grounds naked and with her hands tied after ${getName(names, content.rival)} decided that she needed a lesson for losing yet another spar. Still, ${getName(names, content.rival)} isn't someone that ${character.name} fully hates, but she really wouldn't call her a friend.`,
    );
    return;
  }
  if (content.type === 'friend_encounter_childhood') {
    story.push(
      `Day ${content.cycle}: ${character.name} was always with ${getName(names, content.friend)} during her childhood, growing up in each other's vicinity and hard to split from one another. As they grew up, they find themselves in different circles, but ${character.name} and ${getName(names, content.friend)} always take some time to meet one another.`,
    );
    return;
  }
  if (content.type === 'friend_encounter_adulthood') {
    story.push(
      `Day ${content.cycle}: ${character.name} met ${getName(names, content.friend)} by happenstance while going out. As ${character.name} and ${getName(names, content.friend)} kept talking, they began to form a fast friendship, finding lots in common with one another. Though ${character.name} didn't see ${getName(names, content.friend)} as often as she would have like, she still made time for ${getName(names, content.friend)} as much as she could.`,
    );
    return;
  }
  if (content.type === 'girlfriend_encounter_childhood') {
    story.push(
      `Day ${content.cycle}: ${character.name} was always with ${getName(names, content.girlfriend)} during her childhood, growing up in each other's vicinity and hard to split from one another. As they became adult, ${character.name} finally figures out that what she feels for her friend goes further than friendship, and they both decide to try out dating for a while. The while is stil going, and their relationship is going as strong as ever, as ${getName(names, content.girlfriend)} can attest.`,
    );
    return;
  }
  if (content.type === 'girlfriend_encounter_adulthood') {
    story.push(
      `Day ${content.cycle}: ${character.name} met ${getName(names, content.girlfriend)} by happenstance, but it's immediately clear for both of them that there's a spark. They don't need to spend a lot of time trying out their relationship, as her and ${getName(names, content.girlfriend)} just end up fitting right into one another. They are planning to spend as much of their lives together as they'll be able to.`,
    );
    return;
  }
  if (content.type === 'alone_town_event') {
    story.push(
      `Day ${content.cycle}: ${character.name} leaves for the town at the edge of the Hidden Lands. As she arrives, she finds the town completely deserted but in excellent shape, with both the well, the houses, workshop, training grounds and food resupplier maintained expertly.`,
    );
    return;
  }
  if (content.type === 'group_town_event') {
    story.push(
      `Day ${content.cycle}: ${character.name}, accompanied by ${getNames(names, content.group)}, leaves for the town at the edge of the Hidden Lands. As they arrive, they find the town completely deserted but in excellent shape, with both the well, the houses, workshop, training grounds and food resupplier maintained expertly.`,
    );
    return;
  }
  if (content.type === 'alone_capture_event') {
    story.push(
      `Day ${content.cycle}: While travelling alone towards the Hidden Lands, ${character.name} gets ambushed by ${getEnemies(content.enemies)} and brought to the Dungeon.`,
    );
    return;
  }
  if (content.type === 'group_capture_event') {
    story.push(
      `Day ${content.cycle}: While travelling with ${getNames(names, content.group)} towards the Hidden Lands, ${character.name} and her teammates get ambushed by ${getEnemies(content.enemies)} and brought to the Dungeon.`,
    );
    return;
  }
  assertTypeIsHandled(content, 'backstory event type');
}

/**
 * @param {string[]} story
 * @param {import('../types').StoredCharacterData} character
 * @param {import('../types').CharacterBackstoryEvent} event
 * @param {import('../types').CharacterBackstoryTownEvent} content
 * @param {Record<import('../types').CharacterId, string>} names
 */
function readableTownBackstoryEvent(story, character, event, content, names) {
  if (content.type === 'arrival_base_alone') {
    story.push(`Day ${content.cycle}: ${character.name} arrives in town.`);
    return;
  }
  if (content.type === 'arrival_base_group') {
    story.push(`Day ${content.cycle}: ${character.name} arrives in town with ${getNames(names, content.group)}.`);
    return;
  }
  if (content.type === 'arrival_alone_dungeon') {
    story.push(`Day ${content.cycle}: ${character.name} leaves the Dungeon and gets back to town.`);
    return;
  }
  if (content.type === 'arrival_team_dungeon') {
    story.push(
      `Day ${content.cycle}: ${getCharacterAndNames(character, names, content.group)} all leave the Dungeon and get back to town.`,
    );
    return;
  }
  if (content.type === 'arrival_alone_rescued_people') {
    story.push(
      `Day ${content.cycle}: ${character.name} gets back to town, having successfully saved ${getNames(names, content.rescuees)}.`,
    );
    return;
  }
  if (content.type === 'arrival_team_rescued_people') {
    story.push(
      `Day ${content.cycle}: ${getCharacterAndNames(character, names, content.group)} all get back to town, having successfully saved ${getNames(names, content.rescuees)}.`,
    );
    return;
  }
  if (content.type === 'arrival_team_was_rescued_alone') {
    story.push(
      `Day ${content.cycle}: ${character.name} is safely brought back to town by ${getNames(names, content.group)}, having succesfully escaped the Dungeon.`,
    );
    return;
  }
  if (content.type === 'arrival_team_was_rescued_group') {
    story.push(
      `Day ${content.cycle}: ${character.name}, alongside ${getNames(names, content.rescuees)}, is safely brought back to town by ${getNames(names, content.group)}, having succesfully escaped the Dungeon.`,
    );
    return;
  }
  if (content.type === 'alone_event') {
    story.push(`Day ${content.cycle}: ${character.name} wanders around town.`);
    return;
  }
  if (content.type === 'romance_start') {
    if (content.mode === 'rivalry_ends') {
      story.push(
        `Day ${content.cycle}: After being through so much, that rivalry stops making sense. Instead, ${character.name} and ${getName(names, content.girlfriend)} feel the spark of something new and start a relationship.`,
      );
      return;
    }

    if (content.mode === 'standard') {
      story.push(
        `Day ${content.cycle}: ${character.name} and ${getName(names, content.girlfriend)} take a leap, and get into a relationship.`,
      );
      return;
    }

    assertTypeIsHandled(content.mode, 'backstory event extra condition');
  }
  if (content.type === 'friendship_start') {
    story.push(
      `Day ${content.cycle}: ${character.name}'s friendly conversations with ${getName(names, content.friend)} blossom into a friendship.`,
    );
    return;
  }
  if (content.type === 'rivalry_start_bullies') {
    story.push(
      `Day ${content.cycle}: After another spaarring contest against ${getName(names, content.rival)} that ${character.name} wins, of course, something seems to shift. It's not like ${character.name} isn't still friendly with ${getName(names, content.rival)}, but messing with her is too fun to stop right away, no matter how much ${getName(names, content.rival)} protests!`,
    );
    return;
  }
  if (content.type === 'rivalry_start_is_bullied') {
    story.push(
      `Day ${content.cycle}: After another spaarring contest that ${getName(names, content.rival)} wins again, and that ${character.name} loses, something seems to shift. It's not like ${character.name} can't stand ${getName(names, content.rival)}, but she is starting to get annoyed at how ${getName(names, content.rival)} just seems to enjoy messing with her. It's all ${character.name} can do to not snap at her sometimes, though she starts to be afraid that ${getName(names, content.rival)} would still come out on top even if she did.`,
    );
    return;
  }
  if (content.type === 'romance_event') {
    if (content.kind === 'upgrading_item') {
      story.push(
        `Day ${content.cycle}: While ${character.name} spends some time in the workshop, ${getName(names, content.girlfriend)} comes to visit her. They spend a bit of time chatting, but most of the time, she just enjoys feeling ${getName(names, content.girlfriend)}'s gaze on her as she watches her work.`,
      );
      return;
    }

    if (content.kind === 'watched_upgrade_item') {
      story.push(
        `Day ${content.cycle}: ${character.name} seeks out ${getName(names, content.girlfriend)}, and finds her in the workshop, upgrading an item. She comes in, careful not to disturb ${getName(names, content.girlfriend)}, but she still spends the afternoon alongside her, enjoying watching her work.`,
      );
      return;
    }

    if (content.kind === 'training') {
      story.push(
        `Day ${content.cycle}: While ${character.name} is focusing on her training, ${getName(names, content.girlfriend)} asks if she joins her for practice. It's much easier to learn and to progress once ${getName(names, content.girlfriend)} joins in, and ${character.name} spends the rest of the day energized and in an excellent mood.`,
      );
      return;
    }

    if (content.kind === 'watched_training') {
      story.push(
        `Day ${content.cycle}: ${character.name} enjoys watching ${getName(names, content.girlfriend)} train from afar, as watching her flex and practice is always a great sight. However, it's better up close, so she decides to ask if she can join her. ${character.name} does notice ${getName(names, content.girlfriend)}'s mood improving and her efforts redoubling the moment she joins, but she doesn't comment on it, instead just happy to help her through the paces.`,
      );
      return;
    }

    if (content.kind === 'upgrading_item_watching_training') {
      story.push(
        `Day ${content.cycle}: ${character.name} is spending a lot of time in the workshop, trying to upgrade items for the camp. However, once ${getName(names, content.girlfriend)} starts training outside, ${character.name} can't help but make some extra trips out of the workshop that aren't strictly needed to catch a few glances. ${character.name} feels motivated to get her work done early so she can spend time with ${getName(names, content.girlfriend)} as soon as possible.`,
      );
      return;
    }

    if (content.kind === 'training_watching_upgrading_item') {
      story.push(
        `Day ${content.cycle}: While ${character.name} is focusing on her training, she notices ${getName(names, content.girlfriend)} coming in and out of the workshop she hauled herself in earlier. Each time they glance at each other, ${character.name} feels a pull towards ${getName(names, content.girlfriend)}, but she decides to not bother her and push through her training instead. The faster it's done, the faster they can start spending time together again.`,
      );
      return;
    }

    if (content.kind === 'patrol_round') {
      story.push(
        `Day ${content.cycle}: ${character.name} and ${getName(names, content.girlfriend)} make some rounds around the town, ensuring that everything outside of it is still in order. Hand in hand, they enjoy the strolls, and with nothing to disturb them, it's very much more of a stroll between lovers than a real job for them.`,
      );
      return;
    }

    if (content.kind === 'evening_guard_round') {
      story.push(
        `Day ${content.cycle}: As the evening comes, ${character.name} and ${getName(names, content.girlfriend)} stay awake together a bit later in the evening to ensure nothing happens during the night. They exchange quiet words, each leaning against the other as the remnants of the sunsets are replaced by stars and the various sounds of night. What's often just a chore is improved just by the presence of ${getName(names, content.girlfriend)}, and ${character.name} does consider it a night well-spent.`,
      );
      return;
    }

    if (content.kind === 'gathering_food') {
      story.push(
        `Day ${content.cycle}: ${character.name} and ${getName(names, content.girlfriend)} find themselves with the pretty boring task of handling food supplies for the town. It's often a fairly dull task, mostly inventory and requests for what is missing, but ${getName(names, content.girlfriend)}'s conversation and good mood really help ${character.name} enjoy herself as well. Before ${character.name} even notices, everything is completed, and ${character.name} and ${getName(names, content.girlfriend)} go back hand-in-hand to spend the rest of their evening together.`,
      );
      return;
    }

    if (content.kind === 'town_cleaning') {
      story.push(
        `Day ${content.cycle}: ${character.name} and ${getName(names, content.girlfriend)} end up paired together with the riveting task of cleaning up the town. Or that's what would have happened, but instead ${getName(names, content.girlfriend)}'s funny and upbeat attitude makes ${character.name} more than enjoy her time as well, and in no time, everything is sorted, put away, and overall livable again.`,
      );
      return;
    }

    if (content.kind === 'eating_lively_conversation') {
      story.push(
        `Day ${content.cycle}: At meal time, ${character.name} sits right by ${getName(names, content.girlfriend)} to spend some time conversing. They talk about everything and nothing, but the familiarity and ease of being right next to one another, engaging in conversation, and feeling ${getName(names, content.girlfriend)}'s fond gaze on her each time she apparently does something cute, is all that ${character.name} could wish for.`,
      );
      return;
    }

    if (content.kind === 'friendly_spar') {
      story.push(
        `Day ${content.cycle}: To keep their skills sharp, ${character.name} and ${getName(names, content.girlfriend)} decide to engage in some practice sparring. They exchange blows, bodies clashing against one another in a dance that almost feels synchronized. By the time their training is over, ${character.name} and ${getName(names, content.girlfriend)} can't tell which one of them won, but ${character.name} knows that whichever way the fight went, she certainly feels like a complete winner.`,
      );
      return;
    }

    if (content.kind === 'friendly_spar_with_fun_tops') {
      story.push(
        `Day ${content.cycle}: To keep their skills sharp, ${character.name} and ${getName(names, content.girlfriend)} decide to engage in some practice sparring. ${character.name} seems to keep finding openings after openings in ${getName(names, content.girlfriend)}'s defenses, and she would think her out of her gaze if she didn't notice her flush growing stronger by the minute. After one clean blow that sent her girlfriend's weapon flying and her ass on the ground, ${character.name} decides to push her luck. She leans forward, ordering ${getName(names, content.girlfriend)} to reward her with a kiss for a well-earned victory. The beet-red stuttering that comes out of her girlfriend's mouth right before she gets silenced is exactly what made ${character.name} fall for her.`,
      );
      return;
    }

    if (content.kind === 'friendly_spar_with_fun_bots') {
      story.push(
        `Day ${content.cycle}: To keep their skills sharp, ${character.name} and ${getName(names, content.girlfriend)} decide to engage in some practice sparring. Watching ${getName(names, content.girlfriend)} move around, confident and in control, is too much for ${character.name}'s focus, and she finds herself thrown to the defensive one too many times until she's downed on the ground, staring up at the very confident, very pretty girl she fell for many times. The next words that come out of ${getName(names, content.girlfriend)}'s mouth destroy what's left of her brain, and her stuttered protests as she sees her lean in for a kiss only seem to make ${getName(names, content.girlfriend)}'s smile grow.`,
      );
      return;
    }

    if (content.kind === 'late_afternoon_stroll') {
      story.push(
        `Day ${content.cycle}: Late in the afternoon, ${character.name} and ${getName(names, content.girlfriend)} decide to go on a stroll around the edges of the town. There, hand in hand, they spend some time just enjoying themselves and their surroundings, quiet conversations and comfortable silences following whenever needed. ${character.name} is sure that this afternoon will feature in her memories for a while.`,
      );
      return;
    }

    assertTypeIsHandled(content.kind, 'backstory event extra condition');
  }
  if (content.type === 'friendship_event') {
    if (content.kind === 'upgrading_item') {
      story.push(
        `Day ${content.cycle}: While ${character.name} works on upgrading an item in the workshop, ${getName(names, content.friend)} comes in to have a chat. ${character.name} keeps working as the conversation continues, overall making her job more enjoyable.`,
      );
      return;
    }

    if (content.kind === 'watched_upgrade_item') {
      story.push(
        `Day ${content.cycle}: While ${character.name} strolls around town, she notices ${getName(names, content.friend)} inside of the workshop. She stops by for a chat that lasts a little while, both ${character.name} and ${getName(names, content.friend)} enjoying the conversation.`,
      );
      return;
    }

    if (content.kind === 'training') {
      story.push(
        `Day ${content.cycle}: ${character.name} is working on training her skills, when ${getName(names, content.friend)} proposes to stop by and help. Having another person to help her practice moves and techniques is very helpful, and ${getName(names, content.friend)}'s peppy attitude helps ${character.name} move forward with her training.`,
      );
      return;
    }

    if (content.kind === 'watched_training') {
      story.push(
        `Day ${content.cycle}: ${character.name} notices ${getName(names, content.friend)} training her skills around town, and decides to propose her help. She spends time helping ${getName(names, content.friend)} through her fairly intense program, pushing her as needed and working together on the skills required. As the day progresses and ${character.name} leaves, she feels like ${getName(names, content.friend)} has progressed quite a lot.`,
      );
      return;
    }

    if (content.kind === 'upgrading_item_watching_training') {
      story.push(
        `Day ${content.cycle}: ${character.name} is upgrading an item in the workshop, when ${getName(names, content.friend)} enters, seemingly exhausted by her own training. After a chat where both of them vaguely keep working, they both get remotivated to push each other harder, and ${getName(names, content.friend)} even starts going over her skills right by ${character.name}.`,
      );
      return;
    }

    if (content.kind === 'training_watching_upgrading_item') {
      story.push(
        `Day ${content.cycle}: ${character.name} is working on training her skills, when she notices ${getName(names, content.friend)} that hauled herself up in the workshop. Deciding to commesirate a bit about their respective tasks, it turns out that the short discussion remotivates both herself and ${getName(names, content.friend)} to complete them faster. She even spends a bit of time training right in the workshop before letting ${getName(names, content.friend)} go back to it.`,
      );
      return;
    }

    if (content.kind === 'patrol_round') {
      story.push(
        `Day ${content.cycle}: ${character.name} and ${getName(names, content.friend)} both go on a patrol around town to ensure that nothing is out of place, spending time chatting as they look around.`,
      );
      return;
    }

    if (content.kind === 'evening_guard_round') {
      story.push(
        `Day ${content.cycle}: As night falls, ${character.name} finds herself with ${getName(names, content.friend)} to keep an eye out for threats for the first part of the night. They do keep their conversations fairly quiet, but ${character.name} still enjoys having ${getName(names, content.friend)} to talk with.`,
      );
      return;
    }

    if (content.kind === 'gathering_food') {
      story.push(
        `Day ${content.cycle}: ${character.name} and ${getName(names, content.friend)} find themselves with the pretty boring task of handling food supplies for the town. As they work together on the logistics, inventory, and request part of it all, they do occasionally converse, making the day go faster than either of them really expected.`,
      );
      return;
    }

    if (content.kind === 'town_cleaning') {
      story.push(
        `Day ${content.cycle}: ${character.name} and ${getName(names, content.friend)} end up paired together with the riveting task of cleaning up the town. Still, they make kind of a game off it, seeing who can pack the most boxes or remove the most trash, and by the time everything is done, ${character.name} nearly didn't feel the time pass.`,
      );
      return;
    }

    if (content.kind === 'eating_lively_conversation') {
      story.push(
        `Day ${content.cycle}: During dinner, ${character.name} ends up sitting next to ${getName(names, content.friend)} and has a nice and lively conversation.`,
      );
      return;
    }

    if (content.kind === 'friendly_spar') {
      story.push(`Day ${content.cycle}: ${character.name} has a friendly spar with ${getName(names, content.friend)}.`);
      return;
    }

    assertTypeIsHandled(content.kind, 'backstory event extra condition');
  }
  if (content.type === 'rivalry_event_bullies') {
    if (content.kind === 'upgrading_item') {
      story.push(
        `Day ${content.cycle}: While ${character.name} spends some time in the workshop, she notices ${getName(names, content.rival)} passing by the opened doorway. ${character.name} takes a short moment to grab ${getName(names, content.rival)} by the shoulder, dragging her stuttering ass inside, and locking the door behind them both. ${character.name} unclasps her bracelet, and slaps it around ${getName(names, content.rival)}'s wrist instead before finding a chain and a padlock to keep her stuck with her. While ${getName(names, content.rival)} starts to protest, ${character.name} happily reminds her that she should be a good mana battery and stay silent while the real mage works. The pout that ${getName(names, content.rival)} makes at the words only makes ${character.name} grin harder as she goes back to working with her new, unwilling assistant.`,
      );
      return;
    }

    if (content.kind === 'watched_upgrade_item') {
      story.push(
        `Day ${content.cycle}: As ${character.name} strolls through the town, she notices that ${getName(names, content.rival)} is inside of the workshop, seemingly deep in her work, and can't resist messing with her a bit. She can't approach without being noticed, but she stays casual as ${getName(names, content.rival)} watches her, waiting for the opportune moment to strike. The moment ${getName(names, content.rival)} lets her guard down, ${character.name} grabs a padlock and one of the many chains, and secures ${getName(names, content.rival)}'s magic bracelet to the wall of the workshop. Running away with the key, ${character.name} feels satisfied that she'll help ${getName(names, content.rival)} go above and beyond making her item, unable as she is to do anything else until she deigns freeing her late in the day.`,
      );
      return;
    }

    if (content.kind === 'training') {
      story.push(
        `Day ${content.cycle}: ${character.name} is training her skills when she notices ${getName(names, content.rival)} in her vicinity. She quickly grabs her as sparring partner, and doesn't hesitate to practice a few techniques on ${getName(names, content.rival)} to make sure she got them right. She certainly does, and ${getName(names, content.rival)}'s protest about ${character.name} being "unfair" or "can you please let me go now?" fall into deaf ears as ${character.name} pins her below her.`,
      );
      return;
    }

    if (content.kind === 'watched_training') {
      story.push(
        `Day ${content.cycle}: ${getName(names, content.rival)} walks past the training area and notices ${character.name} trying to train her skill. She can't resist the opportunity to jump in and ask ${getName(names, content.rival)} for a spar, hackling them into using her new techniques. As ${character.name} expected, ${getName(names, content.rival)} can barely execute her new moves, and she quickly finds herself overpowering her and pinning her to the ground. Faced with such a pitiful display, ${character.name} decides that ${getName(names, content.rival)} should focus on the thing she's most likely to be doing, struggling out of grabs. ${character.name}'s comments do seem to motivate ${getName(names, content.rival)} enough that she at least becomes decent at struggling. At least she'll do that pretty well once they finally get in a real fight!`,
      );
      return;
    }

    if (content.kind === 'upgrading_item_watching_training') {
      story.push(
        `Day ${content.cycle}: ${character.name} is upgrading an item in the workshop when she notices ${getName(names, content.rival)} training outside. She quickly grabs her, and makes her practice in the workshop so all of the magic she uses while flailing around doesn't go to waste. The occasional look at ${getName(names, content.rival)}'s progress makes ${character.name} consider on how slow she is, sometimes out loud. At least, she's a helpful assistant in the workshop!`,
      );
      return;
    }

    if (content.kind === 'training_watching_upgrading_item') {
      story.push(
        `Day ${content.cycle}: ${character.name} is upgrading an item in the workshop when she notices ${getName(names, content.rival)} training outside.`,
      );
      return;
    }

    if (content.kind === 'patrol_round') {
      story.push(
        `Day ${content.cycle}: ${character.name} and ${getName(names, content.rival)} end up patrolling the town together. ${character.name} feels an opportunity coming, and can't resist as she suddenly make-believe startles and points at an imaginary spot in the forest. The reaction of ${getName(names, content.rival)} is immediate and hilarious, and ${character.name} laughs as she sees her scramble to grab a weapon before she realizes nothing is there.`,
      );
      return;
    }

    if (content.kind === 'evening_guard_round') {
      story.push(
        `Day ${content.cycle}: ${character.name} and ${getName(names, content.rival)} end up staying awake together during the evening, watching together for threats. As the night goes on, ${character.name} notices ${getName(names, content.rival)}'s tiredness increasing, watching her tilt until she basically lays down against a wall and start to softly snore. ${character.name} spends a few more hours watching over her and the rest of the town, of course, but she doesn't miss mocking ${character.name} the next morning for falling asleep on the job.`,
      );
      return;
    }

    if (content.kind === 'gathering_food') {
      story.push(
        `Day ${content.cycle}: ${character.name} and ${getName(names, content.rival)} find themselves with the pretty boring task of handling food supplies for the town. While ${getName(names, content.rival)} seem to take her task seriously, ${character.name} keeps messing with her counts and making her double-check everything twice as a result. Watching ${getName(names, content.rival)} scrambling to correct herself never gets old for ${character.name}.`,
      );
      return;
    }

    if (content.kind === 'town_cleaning') {
      story.push(
        `Day ${content.cycle}: ${character.name} and ${getName(names, content.rival)} end up paired together with the riveting task of cleaning up the town. While ${getName(names, content.rival)} seems to be doing her best to go through it, ${character.name} decides to mess a bit with her by un-doing some of her work while she isn't watching. It seems to drive ${getName(names, content.rival)} mad, but as soon as she finally figures it out, ${character.name} runs away laughing. They were almost done anyways.`,
      );
      return;
    }

    if (content.kind === 'fight_fair_win') {
      story.push(
        `Day ${content.cycle}: ${character.name} spars against ${getName(names, content.rival)}, and ${character.name} wins. The fight is completely fair, of course, and ${character.name} can't help but feel a bit superior compared to ${getName(names, content.rival)} for winning that one.`,
      );
      return;
    }

    if (content.kind === 'fight_taunts') {
      story.push(
        `Day ${content.cycle}: ${character.name} spars against ${getName(names, content.rival)}, and ${character.name} wins. Some taunting never hurts, even if ${getName(names, content.rival)} started being annoyed stuck below her.`,
      );
      return;
    }

    if (content.kind === 'prank_hides_bag') {
      story.push(
        `Day ${content.cycle}: ${character.name} wants to mess with ${getName(names, content.rival)} for a change, and she hides her bag in some corner of the camp. She has fun watching ${getName(names, content.rival)} look around for it for a while, and she gently ribs her for taking so long once ${getName(names, content.rival)} finally finds it.`,
      );
      return;
    }

    if (content.kind === 'fight_then_prank_hides_bag') {
      story.push(
        `Day ${content.cycle}: ${character.name} spars against ${getName(names, content.rival)}, and ${character.name} wins. ${character.name}, noticing that ${getName(names, content.rival)} is taking some time catching her breath, decides to take ${getName(names, content.rival)}'s bag with her. By the time ${getName(names, content.rival)} gets up and notices, ${character.name} has hid it, and despite ${getName(names, content.rival)}'s protests and complaints, she makes her look for it in the camp.`,
      );
      return;
    }

    if (content.kind === 'fight_cheats') {
      story.push(
        `Day ${content.cycle}: ${character.name} spars against ${getName(names, content.rival)}, but ${character.name} doesn't feel like playing fair this time. After thouroughly overwhelming ${getName(names, content.rival)}, ${character.name} does spend some time ribbing her about not being nearly good enough to handle a few dirty tricks.`,
      );
      return;
    }

    if (content.kind === 'evening_guard_round_prank_ties_up') {
      story.push(
        `Day ${content.cycle}: ${character.name} and ${getName(names, content.rival)} end up staying awake together during the evening, watching together for threats. As the night goes on, ${character.name} sneakily casts a low-power sleeping spell on ${getName(names, content.rival)}, watching her tilt down to the ground and start to softly snore. As soon as her head hits the floor, ${character.name} picks her up and carries her to an empty building. There, she finds a bundle of rope and some strips of cloth, and proceeds to secure her arms and legs before gagging and blindfolding her. Sure, it means that ${character.name} spends the rest of the evening guarding the town alone, but the bewildered look on ${getName(names, content.rival)}'s face the next morning as she tries to figure out what happened, red marks still fresh on her wrists and legs, is all that ${character.name} could have wished for.`,
      );
      return;
    }

    if (content.kind === 'town_cleaning_prank_ties_up') {
      story.push(
        `Day ${content.cycle}: ${character.name} and ${getName(names, content.rival)} end up paired together with the riveting task of cleaning up the town, which bores ${character.name} out of her mind and makes her start looking for something fun to do instead. As they sort things, ${character.name} collects some odds and ends - lengths of ropes, strips of cloth, various things that she thinks she could use. Then, an opportunity falls on her while ${getName(names, content.rival)} focuses on moving one of the big empty boxes. ${character.name} sneaks up on her, grabs her, and with ${getName(names, content.rival)}'s surprise slowing her reflexes, manages to quickly tie her arms and gag her. ${character.name} continues working on securing the rest of ${getName(names, content.rival)}, ignoring her muffled moans. Well, ${character.name} thinks out loud, if ${getName(names, content.rival)} can't dodge a surprise attack, maybe she can learn to escape her binds then, since it seems she'll be doing that a lot? ${character.name} places ${getName(names, content.rival)} in the big box, blindfolds her for completeness' sake, and leaves her to figure it out as she goes back to cleaning what little is left to do. Checking in later in the evening, she finds ${getName(names, content.rival)} drenched in sweat but no closer to escaping, and finally places a knife next to her while mocking her lack of progress. By the time nighttime starts, ${getName(names, content.rival)} is finally out and about the town, but to ${character.name}'s delight it seems like she's too exhausted and goes directly to sleep.`,
      );
      return;
    }

    if (content.kind === 'fight_then_prank_ties_up') {
      story.push(
        `Day ${content.cycle}: ${character.name} spars against ${getName(names, content.rival)}, and ${character.name} wins. ${character.name}, having thouroughly overpowered ${getName(names, content.rival)}, decides to cement her win by taking a length of rope and bundling ${getName(names, content.rival)} up. Sure, ${getName(names, content.rival)} tries to stop her, but there's nothing she can do while ${character.name} works her magic. Still, as ${getName(names, content.rival)}'s protests start to get annoying, a length of fabric helps keep the noise firmly down. Then, once ${getName(names, content.rival)} is truly and securely tied, ${character.name} watches her struggle for a while Unfortunately, it seems like ${getName(names, content.rival)} still hasn't progressed very much, so ${character.name} has to give her a hand by slipping her a knife before leaving her alone to manage her own way out. Surely, ${getName(names, content.rival)} won't mind a few more minutes to train her escape abilities.`,
      );
      return;
    }

    if (content.kind === 'prank_hides_clothes') {
      story.push(
        `Day ${content.cycle}: ${character.name} walks around, and notices ${getName(names, content.rival)} taking a bath while her clothes are left in a heap on her side. While ${getName(names, content.rival)} is distracted, ${character.name} sneakily gathers her clothes and make away with them. Keeping them under her arm, she waits around near the campfire until ${getName(names, content.rival)} finally crawls out of the bath, embarrassed and barely managing to cover herself. ${character.name} grins at her sorry state, but she still gives her her clothing back.`,
      );
      return;
    }

    assertTypeIsHandled(content.kind, 'backstory event extra condition');
  }
  if (content.type === 'rivalry_event_is_bullied') {
    if (content.kind === 'watched_upgrade_item') {
      story.push(
        `Day ${content.cycle}: ${character.name} is minding her own business while passing by the workshop, when the strong hands of ${getName(names, content.rival)} grab her and pull her inside. ${character.name} barely has time to stutter a question before she finds her hand encircled with the workshop's magic bracelet, adorned in typical ${getName(names, content.rival)} fashion with a chain connecting her to the table and a secure-looking padlock stopping her from removing it. ${character.name}'s protests only cause ${getName(names, content.rival)} to demean her further, reminding her that she's just a glorified magic battery at this point. ${character.name} resigns herself to spend the day here alongside ${getName(names, content.rival)}, only allowed to leave once the day is almost over.`,
      );
      return;
    }

    if (content.kind === 'upgrading_item') {
      story.push(
        `Day ${content.cycle}: ${character.name} is spending some time in the workshop, when ${getName(names, content.rival)} comes in and leans against the wall, grinning. ${character.name} throws her a wary look, but all that ${getName(names, content.rival)} seems to be doing is watching her work, seemingly not doing anything for now. Still, ${character.name} should have known better than to go back to focusing on her work. As soon as she stops paying attention to ${getName(names, content.rival)}, she pays for it when she feels something touching her arm, followed by a soft click. Before she can turn around to protest, ${getName(names, content.rival)} is already running out the door with the key to her newly-permanent magic-powered shackled, complete with a chain that binds her to the workshop's table. Calling out after ${getName(names, content.rival)} doesn't do anything, and ${character.name} resigns herself to work until late in the evening when ${getName(names, content.rival)} finally slips her the key.`,
      );
      return;
    }

    if (content.kind === 'watched_training') {
      story.push(
        `Day ${content.cycle}: ${getName(names, content.rival)} walks past the training area, where ${character.name} immediately spots her and grabs her arm. Unwillingly, she's lead into a sparring match where ${character.name} takes the opportunity to demonstrate her mastery of her new techniques. Unfortunately for ${character.name}, ${getName(names, content.rival)} did master them, and ${character.name} can only try to plead and protest after she gets pinned to the ground.`,
      );
      return;
    }

    if (content.kind === 'training') {
      story.push(
        `Day ${content.cycle}: ${character.name} is training her skills when she gets noticed by ${getName(names, content.rival)}. Of course, ${getName(names, content.rival)} quickly asks ${character.name} to demonstrate the techniques she's working on, and ${character.name} tries to apply what she just learned. She gets trounced pretty much immediately, to her opponent's greatest joy, and ${getName(names, content.rival)} decides that ${character.name}'s training should focus on struggling out of grabs since she can't even manage to dodge them. ${character.name}'s pleas to please be let go fall into deaf ears, and although her involuntary struggle practice does somewhat pay off, it isn't without getting a few more taunts from ${getName(names, content.rival)} in return.`,
      );
      return;
    }

    if (content.kind === 'training_watching_upgrading_item') {
      story.push(
        `Day ${content.cycle}: ${character.name} is training near the workshop when ${getName(names, content.rival)} comes out of it and pulls her in. Apparently, ${getName(names, content.rival)} is unhappy that ${character.name} is wasting so much magic learning her spells. To ${character.name}'s reluctant admittance, the magic bracelet does help her train faster, though ${getName(names, content.rival)}'s demeaning comments on ${character.name}'s work as she keeps working on her techniques are not that helpful for sure.`,
      );
      return;
    }

    if (content.kind === 'upgrading_item_watching_training') {
      story.push(
        `Day ${content.cycle}: ${character.name} is training near the workshop when ${getName(names, content.rival)} comes out of it and pulls her in.`,
      );
      return;
    }

    if (content.kind === 'patrol_round') {
      story.push(
        `Day ${content.cycle}: ${character.name} and ${getName(names, content.rival)} end up patrolling the town together. As ${character.name} walks, ${getName(names, content.rival)} suddenly sees something, pointing towards the forest. ${character.name} grabs a weapon, trying to find the threat and what to do about it, when she hears ${getName(names, content.rival)} laughing. Nothing's out there, of course.`,
      );
      return;
    }

    if (content.kind === 'evening_guard_round') {
      story.push(
        `Day ${content.cycle}: ${character.name} and ${getName(names, content.rival)} end up staying awake together during the evening, watching together for threats. As the night goes on, ${character.name} must have fallen asleep, because she wakes up the next morning in the middle of town instead of her own bed. Of course, not only ${getName(names, content.rival)} didn't have the decency to wake her up at the end of their shift, she takes the opportunity to mock her some more.`,
      );
      return;
    }

    if (content.kind === 'gathering_food') {
      story.push(
        `Day ${content.cycle}: ${character.name} and ${getName(names, content.rival)} find themselves with the pretty boring task of handling food supplies for the town. ${character.name} does her best to keep track of what she notes, but every now and then ${getName(names, content.rival)}'s remarks fall just at the wrong time, and she has to start over. Surely, ${getName(names, content.rival)} isn't doing that on purpose... Is she?`,
      );
      return;
    }

    if (content.kind === 'town_cleaning') {
      story.push(
        `Day ${content.cycle}: ${character.name} and ${getName(names, content.rival)} end up paired together with the riveting task of cleaning up the town. The task seems to drag on, and ${character.name} feels like she's redoing the same thing a few times... Until she catches ${getName(names, content.rival)} in the act of undoing some of her work. She can't even properly protest as ${getName(names, content.rival)} runs away to leave her to finish cleaning up alone.`,
      );
      return;
    }

    if (content.kind === 'fight_fair_win') {
      story.push(
        `Day ${content.cycle}: ${character.name} spars against ${getName(names, content.rival)}, and ${character.name} wins. The fight did feel completely fair, but ${character.name} is slightly annoyed at letting ${getName(names, content.rival)} get one over her.`,
      );
      return;
    }

    if (content.kind === 'fight_taunts') {
      story.push(
        `Day ${content.cycle}: ${character.name} spars against ${getName(names, content.rival)}, and ${getName(names, content.rival)} wins. While ${character.name} tries to struggle out of the final grab, ${getName(names, content.rival)} does throw some taunting at her, which gets a bit on her nerves.`,
      );
      return;
    }

    if (content.kind === 'prank_hides_bag') {
      story.push(
        `Day ${content.cycle}: ${character.name} loses her bag, and spends a bunch of time looking for it. It turns out that ${getName(names, content.rival)} hid it somewhere, and no matter how much ${character.name} complains, she isn't saying where. ${character.name} has no choice but to start searching, and of course, ${getName(names, content.rival)} doesn't hesitate to mock ${character.name} for being so oblivious and careless.`,
      );
      return;
    }

    if (content.kind === 'fight_then_prank_hides_bag') {
      story.push(
        `Day ${content.cycle}: ${character.name} spars against ${getName(names, content.rival)}, and ${getName(names, content.rival)} wins. After ${character.name} has taken some time to catching her breath, she stands up, but can't find her bag, and of course ${getName(names, content.rival)} is long-gone too. ${character.name} scours through the camp, and ${getName(names, content.rival)}, who is responsible as always, refuses to say where she hid ${character.name}'s bag. ${character.name} has no choice but to spend at least an hour scouring the camp and looking for it.`,
      );
      return;
    }

    if (content.kind === 'fight_cheats') {
      story.push(
        `Day ${content.cycle}: ${character.name} spars against ${getName(names, content.rival)}, and ${character.name} wins by pulling a dirty move. Defeated, ${character.name} can't do anything to shrug herself of while ${getName(names, content.rival)} gloats about her cheating techniques.`,
      );
      return;
    }

    if (content.kind === 'evening_guard_round_prank_ties_up') {
      story.push(
        `Day ${content.cycle}: ${character.name} and ${getName(names, content.rival)} end up staying awake together during the evening, watching together for threats. As the night goes on, ${character.name} feels herself getting tired, and before she can properly fight it, she falls asleep. When she groggily wakes up, she notices that she can't see, that her limbs are pinned to each other by biting ropes, and that a damp swab of cloth fills her mouth and prevents her from calling out for help. Fearing for the safety of ${getName(names, content.rival)} and the town, ${character.name} struggles mightily, thrashing and searching for a way out, until she manages to find something sharp enough on the ground to cut her binds... And finds herself in one of the town's buildings, alone. Rushing out of the building, she finds the town completely normal, nothing seemingly having happened during the night. She asks ${getName(names, content.rival)} for an explanation, but she just says that yesterday's guard round went fine. ${character.name} sure knows better.`,
      );
      return;
    }

    if (content.kind === 'town_cleaning_prank_ties_up') {
      story.push(
        `Day ${content.cycle}: ${character.name} and ${getName(names, content.rival)} end up paired together with the riveting task of cleaning up the town. To ${character.name}'s surprise, they seem to make fast progress, until the moment where ${character.name} focuses on moving one of the bulky empty boxes they use for storage and gets grabbed from behind without warning. ${character.name} struggles for a moment until she notices that her assaillant is ${getName(names, content.rival)}, and the momentary stun while her brain catches up to the other girl's actions is enough for her arms to get strictly tied behind her. As she opens her mouth to protest, ${getName(names, content.rival)} slips a bundle of cloth in it and quickly secures it, before ${character.name} gets pushed down and secured even further. ${character.name} has to support the mocking remarks of ${getName(names, content.rival)}, and then, to ${character.name}'s horror, she gets carried and placed in the box for "escape training" since, according to ${getName(names, content.rival)}, she's going to find herself doing that a lot if she can't dodge surprise attacks. ${character.name} would remark that she was just surprised ${getName(names, content.rival)} was her assaillant, but she doesn't get the opportunity through her gag, and she gets a blindfold for her troubles anyways. As she hears the footsteps leaving, ${character.name} tries to struggle out of the binds, but the enclosed space and the lack of anything to help her reach the knots makes it impossible. She still tries, for hours, until she finally hears sounds next to her and the lid of the box being opened. Moaning up to get her savior's attention, she hears ${getName(names, content.rival)}'s voice as a reply, mocking her lack of progress and pitiful state but still, mercifully, slipping a knife in her hands. ${character.name} thinks that she could have just untied her, but at this point, she's just happy that she'll get out soon enough. It still takes her a long time before she manages to saw through her binds, finally stumbling out of the box completely exhausted, and falling asleep as soon as she can reach her cot.`,
      );
      return;
    }

    if (content.kind === 'fight_then_prank_ties_up') {
      story.push(
        `Day ${content.cycle}: ${character.name} spars against ${getName(names, content.rival)}, and ${getName(names, content.rival)} wins. Not happy with just overpowering her, ${getName(names, content.rival)} takes a length of rope and start tying ${character.name}'s arms behind her. ${character.name}'s protests fall into deaf ears as ${getName(names, content.rival)} secures her, though at some point, a gag also finds itself between her lips. Once ${getName(names, content.rival)} is done tying her up, and with nothing else to really try to do, ${character.name} starts trying to struggle for a while, but ${getName(names, content.rival)} didn't give her any give nor respite with the binds. Still, it seems like ${getName(names, content.rival)} gets bored after some time, as she slips a knife into ${character.name}'s hand and leaves to do something else. ${character.name} still spends at least an hour undoing ${getName(names, content.rival)}'s ropework, and she feels sore for at least a day afterwards.`,
      );
      return;
    }

    if (content.kind === 'prank_hides_clothes') {
      story.push(
        `Day ${content.cycle}: After ${character.name} gets out of the bath, she notices that all of her clothes have disappeared. She checks and double-checks, but to no avail - there's just nothing around her to cover herself with. After resigning herself to walk through camp half-naked, ${character.name} starts moving, covering herself as best as she can. Of course, ${getName(names, content.rival)} is waiting for her right in the middle with her clothes in hand. ${character.name} is nearly thankful that ${getName(names, content.rival)} doesn't make her pose before giving her clothes back, but not nearly enough to actually get past it, of course.`,
      );
      return;
    }

    assertTypeIsHandled(content.kind, 'backstory event extra condition');
  }
  if (content.type === 'rivalry_end') {
    story.push(
      `Day ${content.cycle}: ${character.name} sets aside her differences with ${getName(names, content.rival)}.`,
    );
    return;
  }
  if (content.type === 'friendship_end') {
    story.push(`Day ${content.cycle}: ${character.name} stops being friends with ${getName(names, content.other)}.`);
    return;
  }
  if (content.type === 'class_changed_event') {
    if (content.new_role === 'adventurer') {
      story.push(
        `Day ${content.cycle}: ${character.name} decides to stop specializing so much. She spends some time to retrain herself into an adventurer.`,
      );
      return;
    }

    if (content.new_role === 'mage') {
      story.push(
        `Day ${content.cycle}: ${character.name}'s affinity with magic makes her focus on casting spells. By the end of the day, she became a mage.`,
      );
      return;
    }

    if (content.new_role === 'rogue') {
      story.push(
        `Day ${content.cycle}: ${character.name}'s affinity with weapons makes her focus on hitting hard. By the end of the day, she became a rogue.`,
      );
      return;
    }

    if (content.new_role === 'warrior') {
      story.push(
        `Day ${content.cycle}: ${character.name}'s affinity with protecting others makes her focus on her defense techniques. By the end of the day, she became a warrior.`,
      );
      return;
    }

    assertTypeIsHandled(content.new_role, 'backstory event extra condition');
  }
  if (content.type === 'upgrade_event') {
    story.push(
      `Day ${content.cycle}: ${character.name} finishes upgrading ${content.item.name} to level ${content.item.level}.`,
    );
    return;
  }
  if (content.type === 'alone_capture_town_corruption') {
    story.push(
      `Day ${content.cycle}: While ${character.name} is hanging out in the town, the Dungeon quietly finishes its corruption of the Hidden Lands. This very day, as mages freely roam the land, they descend on the town, and ${character.name}, the lone inhabitant, quickly gets overpowered and restrained. She is brought back to the Dungeon, to whatever fate awaits her.`,
    );
    return;
  }
  if (content.type === 'group_capture_town_corruption') {
    if (content.kind === 'fought_back') {
      story.push(
        `Day ${content.cycle}: While ${getCharacterAndNames(character, names, content.capturedGroup)} are all hanging out in the town, the Dungeon quietly finishes its corruption of the Hidden Lands. This very day, as mages freely roam the land, they descend on the town, starting to hunt down everyone. ${character.name} notices the incoming approaches, trying to rally for a fight, but she can barely hold a single mage at bay until the sheer number and power of them all finally makes her yield. Still, she did manage to fight back wildly until her binds were just too strict for her to keep going. She is rounded up with ${getNames(names, content.capturedGroup)} in the center of the village, all in equally pitifully bound and gagged. Then, once the mages are satisfied that nobody is let, they're all marched towards the Dungeon, to whatever fate awaits all of them.`,
      );
      return;
    }

    if (content.kind === 'overpowered') {
      story.push(
        `Day ${content.cycle}: While ${getCharacterAndNames(character, names, content.capturedGroup)} are all hanging out in the town, the Dungeon quietly finishes its corruption of the Hidden Lands. This very day, as mages freely roam the land, they descend on the town, starting to hunt down everyone. It's barely even a fight for ${character.name}, who goes down with a single spell and quickly gets restrained and gagged, to the mocking jabs of her assailants. She is rounded up with ${getNames(names, content.capturedGroup)} in the center of the village, all in equally pitifully bound and gagged. Then, once the mages are satisfied that nobody is let, they're all marched towards the Dungeon, to whatever fate awaits all of them.`,
      );
      return;
    }

    if (content.kind === 'ambushed') {
      story.push(
        `Day ${content.cycle}: While ${character.name} is sleeping soundly in town, the Dungeon quietly finishes its corruption of the Hidden Lands. This very day, as mages freely roam the land, they descend on the town, starting to hunt down everyone. ${character.name} gets surprised in the middle of her sleep, finding herself fully bound before she even wakes up. She is rounded up with ${getNames(names, content.capturedGroup)} in the center of the village, all in equally pitifully bound and gagged. Then, once the mages are satisfied that nobody is let, they're all marched towards the Dungeon, to whatever fate awaits all of them.`,
      );
      return;
    }

    assertTypeIsHandled(content.kind, 'backstory event extra condition');
  }
  assertTypeIsHandled(content, 'backstory event type');
}

/**
 * @param {string[]} story
 * @param {import('../types').StoredCharacterData} character
 * @param {import('../types').CharacterBackstoryEvent} event
 * @param {import('../types').CharacterBackstoryExplorationEvent} content
 * @param {Record<import('../types').CharacterId, string>} names
 */
function readableExplorationBackstoryEvent(story, character, event, content, names) {
  if (content.type === 'alone_start_exploration') {
    story.push(`Day ${content.cycle}: ${character.name} makes her way towards the Dungeon on her own.`);
    return;
  }
  if (content.type === 'team_start_exploration') {
    story.push(
      `Day ${content.cycle}: ${getCharacterAndNames(character, names, content.group)} all leave the town towards the Dungeon.`,
    );
    return;
  }
  if (content.type === 'alone_class_changed') {
    story.push(
      `Day ${content.cycle}: As she fights ${getEnemies(content.enemies)}, ${character.name} gets hit by a spell and gets transformed into a ${content.new_role}.`,
    );
    return;
  }
  if (content.type === 'team_class_changed') {
    story.push(
      `Day ${content.cycle}: While fighting alongside ${getNames(names, content.group)} against ${getEnemies(content.enemies)}, a well-placed spell transforms ${character.name} into a ${content.new_role}.`,
    );
    return;
  }
  if (content.type === 'alone_loot') {
    story.push(
      `Day ${content.cycle}: After an arduous fight, ${character.name} finds a level ${content.loot.level} ${content.loot.name}.`,
    );
    return;
  }
  if (content.type === 'team_loot') {
    story.push(
      `Day ${content.cycle}: After disposing of a group of enemies, ${getCharacterAndNames(character, names, content.group)} find a level ${content.loot.level} ${content.loot.name}.`,
    );
    return;
  }
  if (content.type === 'alone_rescue') {
    story.push(
      `Day ${content.cycle}: While pushing through the Dungeon, ${character.name} finds ${getName(names, content.rescuee)} in a cell, wrapped in magic bindings. She takes her along, hoping to bring her back to town safely.`,
    );
    return;
  }
  if (content.type === 'team_rescue') {
    story.push(
      `Day ${content.cycle}: While pushing through the Dungeon, ${getCharacterAndNames(character, names, content.group)} stumble upon a row of cells. Inside, they find ${getName(names, content.rescuee)}, stuck under vicious bindings. They decide to take her along, hoping to protect her long enough to get away.`,
    );
    return;
  }
  if (content.type === 'alone_becomes_rescued') {
    story.push(
      `Day ${content.cycle}: While stuck in a Dungeon cell, ${character.name} gets discovered by ${getName(names, content.rescuer)}, who is silently scouring through the Dungeon. Discreetly, ${getName(names, content.rescuer)} gets her out of the cell, and carries her along as she tries to get her out of the Dungeon.`,
    );
    return;
  }
  if (content.type === 'team_becomes_rescued') {
    story.push(
      `Day ${content.cycle}: While stuck in a Dungeon cell, ${character.name} gets discovered by ${getNames(names, content.group)}. They all manage to get ${character.name} out of the cell but, unable to remove her bindings, try to protect her as they make their way out of the Dungeon.`,
    );
    return;
  }
  if (content.type === 'alone_has_saved_companion') {
    story.push(
      `Day ${content.cycle}: ${character.name} manages to reach town, finally freeing ${getName(names, content.rescuee)} from the Dungeon.`,
    );
    return;
  }
  if (content.type === 'alone_has_saved_companions') {
    story.push(
      `Day ${content.cycle}: ${character.name} manages to reach town, finally freeing ${getNames(names, content.rescuees)} from the Dungeon.`,
    );
    return;
  }
  if (content.type === 'team_has_saved_companion') {
    story.push(
      `Day ${content.cycle}: ${getCharacterAndNames(character, names, content.group)} all manage to reach town, finally freeing ${getName(names, content.rescuee)} from the Dungeon.`,
    );
    return;
  }
  if (content.type === 'team_has_saved_companions') {
    story.push(
      `Day ${content.cycle}: ${getCharacterAndNames(character, names, content.group)} all manage to reach town, finally freeing ${getNames(names, content.rescuees)} from the Dungeon.`,
    );
    return;
  }
  if (content.type === 'alone_becomes_critical') {
    story.push(
      `Day ${content.cycle}: As she fights against ${getEnemies(content.enemies)}, ${character.name} gets hit by a nasty ${content.critical.name} spell, leaving her helpless. Unable to break free, she awaits her fate...`,
    );
    return;
  }
  if (content.type === 'alone_has_critical_companion_event') {
    story.push(
      `Day ${content.cycle}: As ${character.name} and ${getName(names, content.teammate)} fight against ${getEnemies(content.enemies)}, ${getName(names, content.teammate)} gets hit by a ${content.critical.name} spell, leaving her helpless. ${character.name} hopes to be able to get her to safety, and soon.`,
    );
    return;
  }
  if (content.type === 'team_becomes_critical') {
    story.push(
      `Day ${content.cycle}: As ${getCharacterAndNames(character, names, content.group)} fight against ${getEnemies(content.enemies)}, ${character.name} gets hit by a nasty ${content.critical.name} spell, leaving her helpless. Unable to help, she silently encourages ${getNames(names, content.group)}, hoping they'll pull through the rest of the Dungeon.`,
    );
    return;
  }
  if (content.type === 'team_has_critical_companion_event') {
    story.push(
      `Day ${content.cycle}: As ${getCharacterAndNames(character, names, content.group)} fight against ${getEnemies(content.enemies)}, ${getName(names, content.teammate)} gets hit by a ${content.critical.name} spell, leaving her helpless. ${character.name} hopes that she can protect her long enough to get her to safety soon.`,
    );
    return;
  }
  if (content.type === 'alone_has_lost_companion_during_exploration') {
    story.push(
      `Day ${content.cycle}: Pushing further into the Dungeon, ${character.name} and ${getName(names, content.captured)} pass next to an open door. In a rush of movement too fast for ${character.name} to react to, ${getName(names, content.captured)} gets dragged through the door, her screams cut off by the slam of it closing. No matter how hard she tried, ${character.name} can't open the door, and is forced to leave ${getName(names, content.captured)} to her fate.`,
    );
    return;
  }
  if (content.type === 'alone_has_lost_companions_during_exploration') {
    story.push(
      `Day ${content.cycle}: While pushing through the Dungeon, ${character.name} moves forward to check if the way is safe. A noise alerts her, and she turns around just in time to see a team of enemies grabbing ${getNames(names, content.capturedGroup)}. She runs towards them, but not fast enough, leaving the enemies enough time to scatter in all directions. Echoes of muffled screams and fading struggles are all that remains, and, with no way to know which way they went, she is forced to leave all of them behind.`,
    );
    return;
  }
  if (content.type === 'alone_has_lost_companion_during_exit') {
    story.push(
      `Day ${content.cycle}: While trying to get out of the Dungeon, ${character.name} pass near one of the place's many doorways. While she passes it without issue, seeing nobody, ${getName(names, content.captured)} is immediately grabbed by ${getEnemies(content.enemies)} as she passes by it, and a heavy iron grid falls to block the doorway. ${character.name} is forced to watch helplessly as${getName(names, content.captured)} is taken away from her.`,
    );
    return;
  }
  if (content.type === 'alone_has_lost_companions_during_exit') {
    story.push(
      `Day ${content.cycle}: While trying to get out of the Dungeon, ${character.name} makes sure that ${getNames(names, content.capturedGroup)} can esape safely despite their helpless state. However, as soon as she moves away from them, a magic wall rises up, leaving ${character.name} separated from ${getNames(names, content.capturedGroup)}. She can only watch and pound against the wall as, on the other side, ${getEnemies(content.enemies)} simply collect her helpless teammates, dragging them away despite their protests.`,
    );
    return;
  }
  if (content.type === 'team_has_lost_companion_during_exploration') {
    story.push(
      `Day ${content.cycle}: Searching for more survivors, ${getCharacterAndNames(character, names, content.group)} try to keep protecting ${getName(names, content.captured)}. However, even all of them are not enough against the Dungeon. As ${getName(names, content.captured)} falls a bit behind, she screams in distress. ${character.name}'s head barely has time to whip around, seeing ${getName(names, content.captured)} disappearing as she gets dragged away by ${getEnemies(content.enemies)}, and a heavy iron grid falls inbetween ${getCharacterAndNames(character, names, content.group)} and ${character.name}. ${getCharacterAndNames(character, names, content.group)} all try to find a release, but to no avail. Finally, they decide to keep pushing forward, still hoping to be able to rescue ${getName(names, content.captured)}.`,
    );
    return;
  }
  if (content.type === 'team_has_lost_companions_during_exploration') {
    story.push(
      `Day ${content.cycle}: While pushing through the Dungeon, ${getCharacterAndNames(character, names, content.group)} make their way forward, trying to protect ${getNames(names, content.capturedGroup)}. At the turn of a corner, ${getEnemies(content.enemies)} attack them, distracting ${getCharacterAndNames(character, names, content.group)} while ${getNames(names, content.capturedGroup)} all get dragged away in the ruffle. Once the fight is over, ${character.name} and her teammates look around, but it's too late: ${getNames(names, content.capturedGroup)} are truly lost.`,
    );
    return;
  }
  if (content.type === 'team_has_lost_companion_during_exit') {
    story.push(
      `Day ${content.cycle}: While trying to get out of the Dungeon, ${getCharacterAndNames(character, names, content.group)} all make sure sure that ${getName(names, content.captured)} can follow safely despite her helpless state. However, as the door approaches, darkness fall upon them, and ${character.name} tries to flail around without hitting anyone. There's movement, cries and muffled screams, and finally, as the light comes back, ${character.name} can only see that ${getName(names, content.captured)} has disappeared. With no way to save her or chase after her, ${getCharacterAndNames(character, names, content.group)} leave the Dungeon, hoping to come back soon and save her.`,
    );
    return;
  }
  if (content.type === 'team_has_lost_companions_during_exit') {
    story.push(
      `Day ${content.cycle}: While trying to get out of the Dungeon, ${getCharacterAndNames(character, names, content.group)} all make sure sure that ${getNames(names, content.capturedGroup)} can all follow safely despite their helpless state. However, as the door approaches, darkness fall upon them, and ${character.name} tries to flail around without hitting anyone. There's movement, cries and muffled screams, and finally, as the light comes back, ${character.name} can only see that ${getNames(names, content.capturedGroup)} have disappeared. With no way to save them, ${getCharacterAndNames(character, names, content.group)} leave the Dungeon, hoping to come back soon and save them.`,
    );
    return;
  }
  if (content.type === 'alone_becomes_captured') {
    story.push(
      `Day ${content.cycle}: Unable to fight any longer, ${character.name} gets secured and dragged away by ${getEnemies(content.enemies)} to an unknown fate.`,
    );
    return;
  }
  if (content.type === 'alone_becomes_captured_with_companions') {
    story.push(
      `Day ${content.cycle}: As the last of them fall, ${getCharacterAndNames(character, names, content.capturedGroup)} all get secured and dragged away by ${getEnemies(content.enemies)}, seeing their hope of ever escaping dwindling.`,
    );
    return;
  }
  if (content.type === 'team_becomes_captured_alone') {
    story.push(
      `Day ${content.cycle}: Despite the best efforts of ${getNames(names, content.group)}, ${character.name} gets dragged away by ${getEnemies(content.enemies)} to an unknown fate.`,
    );
    return;
  }
  if (content.type === 'team_becomes_captured_with_companions') {
    story.push(
      `Day ${content.cycle}: Despite the best efforts of ${getNames(names, content.group)}, ${getCharacterAndNames(character, names, content.capturedGroup)} gets dragged away by ${getEnemies(content.enemies)} to an unknown fate.`,
    );
    return;
  }
  assertTypeIsHandled(content, 'backstory event type');
}

/**
 * @param {string[]} story
 * @param {import('../types').StoredCharacterData} character
 * @param {import('../types').CharacterBackstoryEvent} event
 * @param {import('../types').CharacterBackstoryCaptureEvent} content
 * @param {Record<import('../types').CharacterId, string>} names
 */
function readableCapturedBackstoryEvent(story, character, event, content, names) {
  if (content.type === 'alone_capture_start') {
    story.push(
      `Day ${content.cycle}: ${character.name} gets dragged away into a lone cell, deep in the depths of the Dungeon.`,
    );
    return;
  }
  if (content.type === 'alone_capture_start_brought_to_girlfriend') {
    story.push(
      `Day ${content.cycle}: ${character.name} gets dragged away into the depths of the Dungeon, into a large, empty cell. However, a few minutes later, ${getName(names, content.girlfriend)} is also brought to her cell. They are left alone to hug and catch up for a while. For whatever reason, it looks like they'll both be able to live in a shared cell together.`,
    );
    return;
  }
  if (content.type === 'group_capture_start') {
    story.push(
      `Day ${content.cycle}: ${getCharacterAndNames(character, names, content.capturedGroup)} all get dragged away into the depths of the Dungeon. They're brought to a prison wing, deep into the depths of the Dungeon, and separated from one another, each given their own, empty cell to live in.`,
    );
    return;
  }
  if (content.type === 'group_capture_start_brought_to_girlfriend') {
    story.push(
      `Day ${content.cycle}: ${getCharacterAndNames(character, names, content.capturedGroup)} all get dragged away into the depths of the Dungeon. They're brought to a prison wing, deep into the depths of the Dungeon, and separated from one another. However, while ${character.name} notices the empty, single-person cell, she is brought instead to a larger cell that can accomodate more people. A few minutes later, ${getName(names, content.girlfriend)} is also brought to her cell from wherever she was kept in the Dungeon. They are left alone to hug and catch up for a while. For whatever reason, it looks like they'll both be able to live in a shared cell together.`,
    );
    return;
  }
  if (content.type === 'group_capture_start_with_girlfriend') {
    story.push(
      `Day ${content.cycle}: ${getCharacterAndNames(character, names, content.capturedGroup.concat(content.girlfriend))} all get dragged away into the depths of the Dungeon. They're brought to a prison wing, deep into the depths of the Dungeon, and her and ${getName(names, content.girlfriend)} get split off from ${getNames(names, content.capturedGroup)}. However, while ${character.name} notices the empty, single-person cell that she and ${getName(names, content.girlfriend)} are made to walk by, they instead are brought to a larger cell that can accomodate the two of them. They are then left alone here. For a sinister reason that they're both vaguely aware of, it looks like they'll both be able to live in a shared cell together.`,
    );
    return;
  }
  if (content.type === 'paired_capture_start_with_girlfriend') {
    story.push(
      `Day ${content.cycle}: ${character.name} and ${getName(names, content.girlfriend)} get dragged away into the depths of the Dungeon. They're brought to a prison wing, deep into the depths of the Dungeon. However, while ${character.name} notices the empty, single-person cells that they get walked by, she and ${getName(names, content.girlfriend)} are instead brought to a larger cell that can accomodate the two of them. They are then left alone here. For a sinister reason that they're both vaguely aware of, it looks like they'll both be able to live in a shared cell together.`,
    );
    return;
  }
  if (content.type === 'girlfriend_reunited_after_capture') {
    story.push(
      `Day ${content.cycle}: One day, a group of mages drag ${character.name} out of her cell, and she already braces herself for a day of torment. However, to her surprise, the mages simply drags ${character.name} into a bigger cell where ${getName(names, content.girlfriend)} already is. She and ${getName(names, content.girlfriend)} fall into a hug, and are left alone to catch up for a while. It looks like they will both live in a shared cell going forward, for whatever reason.`,
    );
    return;
  }
  if (content.type === 'girlfriend_reunited_after_talked') {
    story.push(
      `Day ${content.cycle}: While ${character.name} is still recovering for the earlier ordeal, a small group of mages drag ${character.name} out of her cell. However, to her surprise, the mages simply drags ${character.name} into a bigger cell and leave her there. Moments later, ${getName(names, content.girlfriend)} is brought to her, and ${character.name} starts feeling tears forming in her eyes as ${getName(names, content.girlfriend)} falls into a hug with her. However, as ${getName(names, content.girlfriend)} learns why she was brought here, the tension falls as it feels like nothing will happen. It looks like they will both live in a shared cell going forward, for whatever reason.`,
    );
    return;
  }
  if (content.type === 'girlfriend_reunited_after_other_talked') {
    story.push(
      `Day ${content.cycle}: One day, a group of mages drag ${character.name} out of her cell, and she already braces herself for a day of torment. However, to her surprise, the mages simply drags ${character.name} into a bigger cell where ${getName(names, content.girlfriend)} already is. She and ${getName(names, content.girlfriend)} fall into a hug, and are left alone to catch up for a while, and ${character.name} learns why she was brought here - ${getName(names, content.girlfriend)} talked and told the mages about their relationship. It looks like they will both live in a shared cell going forward, for whatever reason.`,
    );
    return;
  }
  if (content.type === 'alone_wait_event') {
    story.push(
      `Day ${content.cycle}: ${character.name} is left alone in her Dungeon cell, feeling the time pass in loneliness.`,
    );
    return;
  }
  if (content.type === 'alone_play_with_event') {
    story.push(
      `Day ${content.cycle}: ${getEnemies(content.enemies)} decide to bother ${character.name}. She gets tormented for hours in various ways by her captors for no purpose other than their amusements.`,
    );
    return;
  }
  if (content.type === 'alone_magic_event') {
    story.push(
      `Day ${content.cycle}: A group of mages enter ${character.name}'s cell and grab her, placing solid collars and cuffs on her body inscribed with various runes. Then, the mages start running a spell, and ${character.name} feels her mana being drained. She is used as a magic battery in various experiments for the rest of the day.`,
    );
    return;
  }
  if (content.type === 'alone_drain_event') {
    story.push(
      `Day ${content.cycle}: ${character.name} spends the day getting her experience siphoned out by a mage to upgrade the provided ${content.equipment.name}.`,
    );
    return;
  }
  if (content.type === 'alone_drain_into_girlfriend_event') {
    story.push(
      `Day ${content.cycle}: While ${character.name} is still mourning the loss of ${getName(names, content.girlfriend)}, a mage enters the cell, bringing the too-familiar ${content.equipment.name} with her. Mercilessly, the mage drains ${character.name}'s experience into the item that ${getName(names, content.girlfriend)} was turned into, apparently with the sole goal of upgrading it.`,
    );
    return;
  }
  if (content.type === 'companion_play_with_event') {
    story.push(
      `Day ${content.cycle}: ${getEnemies(content.enemies)} come in to drag ${character.name} to a bigger cell, where she joins ${getName(names, content.captured)}. There, they all get subjected to hours of play and humiliation for the amusement of their captors.`,
    );
    return;
  }
  if (content.type === 'companion_magic_event') {
    story.push(
      `Day ${content.cycle}: A group of mages enter to drag ${character.name} to a bigger cell, where she joins ${getName(names, content.captured)}. Then, the mages start placing solid collars and cuffs on her and ${getName(names, content.captured)}'s body inscribed with various runes. Then, the mages start running a spell, and ${character.name} feels her mana being drained. She and ${getName(names, content.captured)} are both used as a magic battery in various experiments for the rest of the day.`,
    );
    return;
  }
  if (content.type === 'girlfriend_wait_event') {
    story.push(
      `Day ${content.cycle}: ${character.name} and ${getName(names, content.girlfriend)} are both left on their own in their shared Dungeon cell, feeling the time pass while exchanging some whispered words. At least, they have each other.`,
    );
    return;
  }
  if (content.type === 'girlfriend_play_with_event') {
    if (content.state === 'secret') {
      story.push(
        `Day ${content.cycle}: ${getEnemies(content.enemies)} come in to drag ${character.name} to a bigger cell, where to her surprise, she joins ${getName(names, content.girlfriend)}. Unfortunately, the surprise is quickly replaced by dread, and she and ${getName(names, content.girlfriend)} both get subjected to hours of play and humiliation for the amusement of their captors, made worse by their captor's notice of both of their anguish, and the fact that neither she nor ${getName(names, content.girlfriend)} is willing to tell ${getEnemies(content.enemies)} why she wants them to stop.`,
      );
      return;
    }

    if (content.state === 'known') {
      story.push(
        `Day ${content.cycle}: ${getEnemies(content.enemies)} decide to enter ${character.name} and ${getName(names, content.girlfriend)}'s cell and bother them. One after the other, they get tormented for hours in various ways while the other has to watch, their captors's ear deaf to their pleas. Once ${getEnemies(content.enemies)} get bored, they leave her and ${getName(names, content.girlfriend)} alone. It seems that this whole ordeal had no purpose other than their captor's amusements.`,
      );
      return;
    }

    assertTypeIsHandled(content.state, 'backstory event extra condition');
  }
  if (content.type === 'girlfriend_magic_event') {
    if (content.state === 'secret') {
      story.push(
        `Day ${content.cycle}: A group of mages enter to drag ${character.name} to a bigger cell, where to her surprise, she joins ${getName(names, content.girlfriend)}. Unfortunately, the surprise is quickly replaced by dread, as the mages start placing solid collars and cuffs on her and ${getName(names, content.girlfriend)}'s body inscribed with various runes. Then, the mages start running a spell, and ${character.name} feels her mana being drained. She and ${getName(names, content.girlfriend)} are both used as a magic battery in various experiments for the rest of the day.`,
      );
      return;
    }

    if (content.state === 'known') {
      story.push(
        `Day ${content.cycle}: A group of mages enter ${character.name} and ${getName(names, content.girlfriend)}'s cell. The mages then start placing solid collars and cuffs on her and ${getName(names, content.girlfriend)}'s body inscribed with various runes. Then, the mages start running a spell, and ${character.name} feels her mana being drained. She and ${getName(names, content.girlfriend)} are both used as a magic battery in various experiments for the rest of the day.`,
      );
      return;
    }

    assertTypeIsHandled(content.state, 'backstory event extra condition');
  }
  if (content.type === 'alone_interrogate_event_stayed_silent') {
    story.push(
      `Day ${content.cycle}: A group of mages enter ${character.name}'s cell, and without a word, drag her to a well-furnished interrogation room. There, one by one, they question, threaten, and pump her for any information she might have, while the glow of a powerful truth spell constricts her neck each time she refuses to answer. The ordeal lasts for hours, ${character.name} barely able to hold on, but finally, the mages start to get bored and bring her back to her prison cell. ${character.name} finally breathes, having managed to not say anything of value. Or at least this time...`,
    );
    return;
  }
  if (content.type === 'alone_interrogate_event_talked') {
    story.push(
      `Day ${content.cycle}: A group of mages enter ${character.name}'s cell, and without a word, drag her to a well-furnished interrogation room. There, one by one, they question, threaten, and pump her for any information she might give to her captors. Under the pressure, ${character.name} breaks down and starts talking, telling them all of her history, knowledge, and every single detail they ask. The mages, satisfied with their results, bring her back to her cell.`,
    );
    return;
  }
  if (content.type === 'alone_interrogate_event_denounced_girlfriend') {
    story.push(
      `Day ${content.cycle}: A group of mages enter ${character.name}'s cell, and without a word, drag her to a well-furnished interrogation room.There, one by one, they question, threaten, and pump her for any information she might give to her captors. ${character.name} tries to hold on as long as she can, but eventually, it is too much, she breaks and starts answering their oddly specific questions. Finally, she talks about ${getName(names, content.girlfriend)}, and as she sees the mages' eyes light up, she knows she made a terrible mistake. Still, the deed is done, and the mages, more than satisfied with their results, drag ${character.name} back to her lonesome cell.`,
    );
    return;
  }
  if (content.type === 'companion_interrogate_event_stayed_silent') {
    story.push(
      `Day ${content.cycle}: A group of mages enter ${character.name}'s cell, and without a word, drag her to a well-furnished interrogation room. There, she joins ${getName(names, content.captured)} for an intense interrogation session. Relentlessly, the mages question, threaten, and pump her and ${getName(names, content.captured)} for any information they might have, while the glow of a powerful truth spell constricts both of their neck each time either of them refuses to answer. The ordeal lasts for hours, ${character.name} barely able to hold on, and she sees ${getName(names, content.captured)} at the verge of breaking as well. But, finally, the mages start to get bored and bring ${character.name} back to her prison cell. ${character.name} finally breathes, having managed to not say anything of value. Or at least this time...`,
    );
    return;
  }
  if (content.type === 'companion_interrogate_event_self_denounced_own_girlfriend') {
    story.push(
      `Day ${content.cycle}: A group of mages enter ${character.name}'s cell, and without a word, drag her to a well-furnished interrogation room. There, she joins ${getName(names, content.captured)} for an intense interrogation session. Relentlessly, the mages question, threaten, and pump her and ${getName(names, content.captured)} for any information they might have, while the glow of a powerful truth spell constricts both of their neck each time either of them refuses to answer. Under pressure of seeing ${getName(names, content.captured)} getting tortured, ${character.name} breaks she breaks and starts answering their oddly specific questions. Finally, she talks about ${getName(names, content.girlfriend)}, to ${getName(names, content.captured)}'s horror, and as she sees the mages' eyes light up, she knows she made a terrible mistake. Still, the deed is done, and the mages, more than satisfied with their results, drag both ${getName(names, content.captured)} and ${character.name} away, and ${character.name} is sent back to her lonesome cell, dreading the consequences of her speech.`,
    );
    return;
  }
  if (content.type === 'companion_interrogate_event_self_denounced_other_girlfriend') {
    story.push(
      `Day ${content.cycle}: A group of mages enter ${character.name}'s cell, and without a word, drag her to a well-furnished interrogation room. There, she joins ${getName(names, content.captured)} for an intense interrogation session. Relentlessly, the mages question, threaten, and pump her and ${getName(names, content.captured)} for any information they might have, while the glow of a powerful truth spell constricts both of their neck each time either of them refuses to answer. Under the pressure of getting tortured so much, ${character.name} breaks and starts answering their oddly specific questions, despite ${getName(names, content.captured)}'s protests which quickly get her gagged and silenced. While it seems like she doesn't have much of value to add herself, she does finally say something that interests the mages. She lets it slip that ${getName(names, content.targetGirlfriend)} is ${getName(names, content.captured)}'s girlfriend, and the mages' eyes immediately start to glean. ${getName(names, content.captured)} tries to shout and groan, shaking her head to deny it, but too late. Both ${getName(names, content.captured)} and ${character.name} are dragged away, both in their own cell, and ${character.name} is left to wonder what horrible fate she doomed ${getName(names, content.captured)} and ${getName(names, content.targetGirlfriend)} to.`,
    );
    return;
  }
  if (content.type === 'companion_interrogate_event_other_denounced_self_girlfriend') {
    story.push(
      `Day ${content.cycle}: A group of mages enter ${character.name}'s cell, and without a word, drag her to a well-furnished interrogation room. There, she joins ${getName(names, content.captured)} for an intense interrogation session. Relentlessly, the mages question, threaten, and pump her and ${getName(names, content.captured)} for any information they might have, while the glow of a powerful truth spell constricts both of their neck each time either of them refuses to answer. Under the pressure of getting tortured so much, and to ${character.name}'s horror, ${getName(names, content.captured)} breaks and starts answering their oddly specific questions. As soon as ${character.name} tries to protest or stop them, one of the mages strictly gag her and she can just look in horror. While the interrogation continues and ${getName(names, content.captured)} answers more and more, she lets one name slip, and ${character.name}'s blood freezes in her veins. ${getName(names, content.captured)} let the mages know about ${getName(names, content.girlfriend)}, and the mages' eyes immediately start to glean. ${character.name} tries to shout in her gag, to divest attention, but too late. Both ${getName(names, content.captured)} and ${character.name} are dragged away, both in their own cell, and ${character.name} is left to wonder what horrible fate awaits her and ${getName(names, content.girlfriend)}.`,
    );
    return;
  }
  if (content.type === 'companion_interrogate_event_other_denounced_other_girlfriend') {
    story.push(
      `Day ${content.cycle}: A group of mages enter ${character.name}'s cell, and without a word, drag her to a well-furnished interrogation room. There, she joins ${getName(names, content.captured)} for an intense interrogation session. Relentlessly, the mages question, threaten, and pump her and ${getName(names, content.captured)} for any information they might have, while the glow of a powerful truth spell constricts both of their neck each time either of them refuses to answer. At some point, as ${character.name} keeps getting tortured harder and harder, she feels a wave of relief at it all stops. However, the relief is replaced by dread as she hears ${getName(names, content.captured)} starts answering the mages' oddly specific questions. To ${character.name}'s increasingly panicked ears, ${getName(names, content.captured)} talks about her girlfriend ${getName(names, content.targetGirlfriend)}, and she sees the mages' eyes light up. ${character.name} can't do much and, the deed is done, and the mages, more than satisfied with their results, drag both ${getName(names, content.captured)} and ${character.name} away, and ${character.name} is sent back to her lonesome cell, wondering what the fate of ${character.name} and ${getName(names, content.targetGirlfriend)} will be after all of this.`,
    );
    return;
  }
  if (content.type === 'girlfriend_interrogate_event_stayed_silent') {
    story.push(
      `Day ${content.cycle}: A group of mages enter ${character.name}'s cell, and without a word, drag her to a well-furnished interrogation room. There, to her barely concealed surprise, she joins ${getName(names, content.girlfriend)} for an intense interrogation session. Relentlessly, the mages question, threaten, and pump her and ${getName(names, content.girlfriend)} for any information they might have, while the glow of a powerful truth spell constricts both of their neck each time either of them refuses to answer. The ordeal lasts for hours, ${character.name} barely able to hold on and even worse, having to stop herself from pleading for ${getName(names, content.girlfriend)}'s life each time the mage focuses back on her. But, finally, the mages start to get bored and drag ${character.name} and ${getName(names, content.girlfriend)} away from each other, and back to their own cell. They exchange one brief look at they part, but ${character.name} finally breathes, having managed to not say anything of value. Or at least this time...`,
    );
    return;
  }
  if (content.type === 'girlfriend_interrogate_event_broke') {
    story.push(
      `Day ${content.cycle}: A group of mages enter ${character.name}'s cell, and without a word, drag her to a well-furnished interrogation room. There, to her barely concealed surprise, she joins ${getName(names, content.girlfriend)} for an intense interrogation session. Relentlessly, the mages question, threaten, and pump her and ${getName(names, content.girlfriend)} for any information they might have, while the glow of a powerful truth spell constricts both of their neck each time either of them refuses to answer. The ordeal lasts for hours, ${character.name} barely able to hold on, but at some point seeing what the mages do to ${getName(names, content.girlfriend)} becomes too much. As she starts to blabble some half-answers hoping to lessen both her and her girlfriend's ordeal, she is surprised to hear ${getName(names, content.girlfriend)} also saying things, completing what she starts saying. The torture finally stops, and the mages, surprisingly caring, let her and ${getName(names, content.girlfriend)} fall into each other's arms as they tell each one and each other's life story in excruciating details. ${character.name} still doesn't miss the way the mages' eyes did light up, which presages nothing good. Still, once the interrogation is done, the mages don't bring ${character.name} nor ${getName(names, content.girlfriend)} back to their own cell, but to a bigger, shared cell instead. Exhausted, ${character.name} and ${getName(names, content.girlfriend)} quickly fall asleep, hugging each other in the dark cell of the Dungeon.`,
    );
    return;
  }
  if (content.type === 'transmutation_prepare_1_alone') {
    story.push(
      `Day ${content.cycle}: ${character.name} gets the visit of a mage. The mage spends time measuring her various sizes and shapes in plenty of ways, ${character.name} forced to stay silent by a spell and unable to resist her invasive touch. Then, the mage leaves, but doesn't remove any of her spells. ${character.name} is forced to spend the rest of her day mute and bound by the remaining spells.`,
    );
    return;
  }
  if (content.type === 'transmutation_prepare_2_alone') {
    story.push(
      `Day ${content.cycle}: The mage that tormented ${character.name} comes back. This time, after another brief scuffle and spell thrown, the mage directly strips ${character.name} of most of her clothes, and starts drawing lines on her body using charcoal. With horror, ${character.name} remembers the transmutation rituals, one of the many things that the Dungeon is infamous for, and can already know what fate she can expect. After the mage leaves, taking ${character.name}'s clothes with her, ${character.name} spends a few hours trying to rub the charcoal from her skin, but the marks refuse to fade.`,
    );
    return;
  }
  if (content.type === 'transmutation_prepare_3_alone') {
    story.push(
      `Day ${content.cycle}: The mage that wants to transmute ${character.name} comes back once again. ${character.name} tries all she can to squirm away, however, there is nowhere to escape to. This time, the mage traces back over the previous charcoal lines with purple ink, coping a feel more than once as she does. ${character.name} feels her dread rise the whole time, and the departure of the mage after a few hours only increases it further.`,
    );
    return;
  }
  if (content.type === 'transmutation_prepare_1_in_front_of_girlfriend') {
    story.push(
      `Day ${content.cycle}: ${character.name} and ${getName(names, content.girlfriend)} gets the visit of two mages. After a brief scuffle where the mages throw a spell on both of them to keep them silent and bound, a long time ensues where her and ${getName(names, content.girlfriend)} gets measured in plenty of ways, their various sizes and shapes meticulously recorded. After some time, and to ${getName(names, content.girlfriend)}'s obvious disapproval but utter powerlessness, the mages clearly start to focus on ${character.name} more, who is unable to resist their increasingly invasive touches. Still, after a while, the mages leave, seeemingly forgetting to remove any of their spells. ${character.name} manages to crawl near ${getName(names, content.girlfriend)}, and they're forced to wait the rest of their day mute and bound by the spells.`,
    );
    return;
  }
  if (content.type === 'transmutation_girlfriend_is_prepared_1') {
    story.push(
      `Day ${content.cycle}: ${character.name} and ${getName(names, content.girlfriend)} gets the visit of two mages. After a brief scuffle where the mages throw a spell on both of them to keep them silent and bound, a long time ensues where her and ${getName(names, content.girlfriend)} gets measured in plenty of ways, their various sizes and shapes meticulously recorded. After some time, and to ${character.name}'s obvious disapproval but utter powerlessness, the mages clearly start to focus on ${getName(names, content.girlfriend)} more. ${character.name} is forced to watch while they start touching her in increasingly more invasive ways, and her anguish at ${getName(names, content.girlfriend)}'s discomfort doesn't change anything. Still, after a while, the mages leave, seeemingly forgetting to remove any of their spells. ${character.name} struggles to crawl near ${getName(names, content.girlfriend)} to comfort her as much as she can, and while they still have to spend the rest of their day mute and bound by the spells, ${character.name} hopes she could soothe ${getName(names, content.girlfriend)} at least a bit.`,
    );
    return;
  }
  if (content.type === 'transmutation_prepare_2_in_front_of_girlfriend') {
    story.push(
      `Day ${content.cycle}: The mages that tormented ${character.name} and ${getName(names, content.girlfriend)} comes back. This time, after another brief scuffle and spell thrown, the mages directly strip ${character.name} of most of her clothes under the distressed gaze of ${getName(names, content.girlfriend)}, and they start drawing lines on her body using charcoal. A vague memory hits ${character.name} as she remembers the transmutation rituals, one of the many things that the Dungeon is infamous for. She exchanges a terrified look with ${getName(names, content.girlfriend)}, and it seems like both of them already know what fate ${character.name} can expect. After a while, the mages leave, taking ${character.name}'s clothes with her. ${character.name} and ${getName(names, content.girlfriend)} both spend a few hours trying to rub the charcoal from ${character.name}'s skin, but the marks refuse to fade.`,
    );
    return;
  }
  if (content.type === 'transmutation_girlfriend_is_prepared_2') {
    story.push(
      `Day ${content.cycle}: Two mages enter, and ${character.name} immediately recognizes them as the ones that tormented ${getName(names, content.girlfriend)}, throwing them a look she wishes could kill. However, another brief scuffle and spell thrown is all it takes to render both ${character.name} and ${getName(names, content.girlfriend)} powerless, and the mages spend no time to go back to tormenting ${character.name} specifically, stripping her of most of her clothes while ${character.name} can't even protect her. Then, the mages start drawing lines on ${getName(names, content.girlfriend)}'s body using charcoal. After a few moments, ${character.name} remembers the transmutation rituals, one of the many things that the Dungeon is infamous for. Panicking, she looks towards ${getName(names, content.girlfriend)}'s eyes, hoping against all reason that her guess isn't correct, but the look that ${getName(names, content.girlfriend)} throws back all but confirms it. After a while, the mages leave, taking ${character.name}'s clothes with her. ${character.name} helps cover ${getName(names, content.girlfriend)} as much as she can, and she insists to try to rub the charcoal from ${character.name}'s skin, but no matter what she does, the marks refuse to fade.`,
    );
    return;
  }
  if (content.type === 'transmutation_prepare_3_in_front_of_girlfriend') {
    story.push(
      `Day ${content.cycle}: The two mages that wants to transmute ${character.name} come back once again, and ${character.name} jumps as ${getName(names, content.girlfriend)} immediately raises to fight them. However, in a single spell, they throw ${getName(names, content.girlfriend)} back against their shared cot where she's left to struggle. As for her, ${character.name} tries all she can to squirm away from them, but there is nowhere to escape to, and the two mages are practiced. This time, the mages trace back over the previous charcoal lines adorning ${character.name}'s body with purple ink, coping a feel more than once as their hands roam around to ${character.name}'s anguish. ${character.name} feels her dread rise the whole time, and the departure of the mages after a few hours only increases it further. Even hugging ${getName(names, content.girlfriend)} barely provides relief.`,
    );
    return;
  }
  if (content.type === 'transmutation_girlfriend_is_prepared_3') {
    story.push(
      `Day ${content.cycle}: The two mages that wants to transmute ${getName(names, content.girlfriend)} come back once again, and ${character.name} immediately raises to fight them. However, it takes a single spell to throw ${character.name} back against their shared cot, and while she unsuccessfully tries to struggle back, ${getName(names, content.girlfriend)} on her side tries all she can to squirm away from them. However, there is nowhere to escape to, and the two mages are practiced. This time, the mages trace back over the previous charcoal lines with purple ink, their hands often wandering over ${getName(names, content.girlfriend)}'s body to cope a feel and making ${character.name} livid. ${character.name} feels her annoyance, dread, and hopelessness rise the whole time, and the departure of the mages after a few hours only increases it further. Even hugging ${getName(names, content.girlfriend)} barely provides relief.`,
    );
    return;
  }
  if (content.type === 'transmutation_girlfriend_is_executed') {
    story.push(
      `Day ${content.cycle}: The day of the ritual is finally here. The two mages come back, one restraining ${character.name} while the other takes hold of ${getName(names, content.girlfriend)}. They both get dragged, ${character.name} thrashing and screaming, and ${getName(names, content.girlfriend)} stark naked and resigned, to a giant room in the middle of the Dungeon. There, she can see ${getName(names, content.girlfriend)} being dragged and restrained in the middle of the transmutation circle, while ${character.name} herself is brought inside of an iron cage overseeing the scene by the other mage. The ritual attracts a bunch of people, mages coming from the whole Dungeon. As the ritual completes, ${character.name} calls up to ${getName(names, content.girlfriend)} to assure her that she loves her. However, in a flash, ${getName(names, content.girlfriend)} gets transformed into a ${content.equipment.name}.The ritual continues for hours after that, ${character.name} getting repeatedly drained of her experience as more and more of it gets poured into the inert ${content.equipment.name}. Once it's done, ${character.name} feeling lesser both in mind and spirit, she is brought back to a lone cell in the depths of the Dungeon to wallow on her own fate.`,
    );
    return;
  }
  assertTypeIsHandled(content, 'backstory event type');
}

/**
 * @param {string[]} story
 * @param {import('../types').StoredCharacterData} character
 * @param {import('../types').CharacterBackstoryEvent} event
 * @param {import('../types').CharacterBackstoryTransmutedEvent} content
 * @param {Record<import('../types').CharacterId, string>} names
 */
function readableTransmutedBackstoryEvent(story, character, event, content, names) {
  if (content.type === 'transmutation_execute_alone') {
    story.push(
      `Day ${content.cycle}: The day of the ritual is finally here. The mage comes to fetch ${character.name} in her cell, bringing her to a giant room at the center of the Dungeon. There, she is restrained, alone and naked, in the middle of the transmutation circle, surrounded by mages coming from the whole Dungeon. The ritual completes, and in a flash, she gets transformed into a lv. ${content.equipment.level} ${content.equipment.name}, her soul forever trapped in the ${content.equipment.name}'s gem.`,
    );
    return;
  }
  if (content.type === 'transmutation_execute_in_front_of_girlfriend') {
    story.push(
      `Day ${content.cycle}: The day of the ritual is finally here. The two mages come back, one restraining ${getName(names, content.girlfriend)} while the other takes hold of ${character.name}. They both get dragged, ${getName(names, content.girlfriend)} thrashing and screaming, and ${character.name} stark naked and resigned, to a giant room in the middle of the Dungeon. There, ${character.name} is restrained in the middle of the transmutation circle, while the mages take ${getName(names, content.girlfriend)} and bring her inside of an iron cage overseeing the scene. The ritual attracts a bunch of people, mages coming from the whole Dungeon. The ritual completes, and ${character.name} can barely hear ${getName(names, content.girlfriend)} start to call something, but in a flash, ${character.name} gets transformed into a level ${content.equipment.level} ${content.equipment.name}, her soul forever trapped in the ${content.equipment.name}'s gem. She never even heard the last words her girlfriend said to her.`,
    );
    return;
  }
  if (content.type === 'alone_event') {
    story.push(`Day ${content.cycle}: ${character.name} can barely feel or think anything.`);
    return;
  }
  if (content.type === 'drain_upgrade_event') {
    story.push(
      `Day ${content.cycle}: ${character.name} feels herself getting infused by the essence of ${getName(names, content.teammate)}. She is now a level ${content.equipment.level} ${content.equipment.name}.`,
    );
    return;
  }
  if (content.type === 'enemy_use_event') {
    story.push(
      `Day ${content.cycle}: ${character.name} is fairly sure that she gets used in a fight by ${getEnemies(content.enemies)}.`,
    );
    return;
  }
  if (content.type === 'enemy_group_use_event') {
    story.push(
      `Day ${content.cycle}: ${character.name} is fairly sure that she gets used alongside ${getNames(names, content.capturedGroup)} in a fight by ${getEnemies(content.enemies)}.`,
    );
    return;
  }
  if (content.type === 'team_upgrade_event') {
    story.push(
      `Day ${content.cycle}: ${character.name} feels herself getting upgraded to a stronger item by ${getName(names, content.teammate)}. She is now a level ${content.equipment.level} ${content.equipment.name}.`,
    );
    return;
  }
  if (content.type === 'team_use_event') {
    story.push(
      `Day ${content.cycle}: ${character.name} gets used by ${getName(names, content.teammate)} during an expedition.`,
    );
    return;
  }
  if (content.type === 'team_group_use_event') {
    story.push(
      `Day ${content.cycle}: ${character.name} gets used by ${getName(names, content.teammate)} during an expedition alongside ${getNames(names, content.capturedGroup)}.`,
    );
    return;
  }
  if (content.type === 'turned_into_dust_event') {
    story.push(
      `Day ${content.cycle}: ${character.name} feels herself being grinded down into dust that will be used later when upgrading other items. It's the end of the line for ${character.name}.`,
    );
    return;
  }
  assertTypeIsHandled(content, 'backstory event type');
}
