import { assertTypeIsHandled } from '../utils.mjs';

/**
 * @param { 'damage_small' | 'damage_medium' | 'damage_large' | 'damage_very_large' } attack_id
 * @returns {import("../types").InGameEnemySkill}
 */
export function generateDamagingAttack(attack_id) {
  if (attack_id === 'damage_small') {
    return {
      id: attack_id,
      characteristics: {
        id: attack_id,
        name: 'Attack',
        target: { type: 'random_enemy' },
        action: { type: 'damage', damage_hp: 5 },
        cost_data: {
          cooldown: 50,
          mana_percent: 0,
          mana: 0,
        },
      },
      condition: { type: 'target_not_critical' },
      cooldown: { remaining_ticks: 0 },
    };
  }
  if (attack_id === 'damage_medium') {
    return {
      id: attack_id,
      characteristics: {
        id: attack_id,
        name: 'Attack',
        target: { type: 'random_enemy' },
        action: { type: 'damage', damage_hp: 10 },
        cost_data: {
          cooldown: 50,
          mana_percent: 0,
          mana: 0,
        },
      },
      condition: { type: 'target_not_critical' },
      cooldown: { remaining_ticks: 0 },
    };
  }
  if (attack_id === 'damage_large') {
    return {
      id: attack_id,
      characteristics: {
        id: attack_id,
        name: 'Attack',
        target: { type: 'random_enemy' },
        action: { type: 'damage', damage_hp: 40 },
        cost_data: {
          cooldown: 50,
          mana_percent: 0,
          mana: 0,
        },
      },
      condition: { type: 'target_not_critical' },
      cooldown: { remaining_ticks: 0 },
    };
  }
  if (attack_id === 'damage_very_large') {
    return {
      id: attack_id,
      characteristics: {
        id: attack_id,
        name: 'Attack',
        target: { type: 'random_enemy' },
        action: { type: 'damage', damage_hp: 2000 },
        cost_data: {
          cooldown: 10 * 20,
          mana_percent: 0,
          mana: 0,
        },
      },
      condition: { type: 'target_not_critical' },
      cooldown: { remaining_ticks: 10 * 20 },
    };
  }
  assertTypeIsHandled(attack_id, 'attack id');
}

/**
 * @param { 'drain_magic_small_on_critical' | 'drain_magic_large_on_critical' | 'drain_magic_on_game_over' | 'drain_exp_on_game_over' } attack_id
 * @returns {import("../types").InGameEnemySkill}
 */
export function generateDrainingEnemyAttack(attack_id) {
  if (attack_id === 'drain_magic_small_on_critical') {
    return {
      id: attack_id,
      characteristics: {
        id: attack_id,
        name: 'Attack',
        target: { type: 'random_enemy' },
        action: { type: 'drain_mana', damage_mp: 5, drain_ratio: 1.0 },
        cost_data: {
          cooldown: 50,
          mana_percent: 0,
          mana: 0,
        },
      },
      condition: { type: 'target_critical' },
      cooldown: { remaining_ticks: 0 },
    };
  }
  if (attack_id === 'drain_magic_large_on_critical') {
    return {
      id: attack_id,
      characteristics: {
        id: attack_id,
        name: 'Attack',
        target: { type: 'random_enemy' },
        action: { type: 'drain_mana', damage_mp: 30, drain_ratio: 1.0 },
        cost_data: {
          cooldown: 50,
          mana_percent: 0,
          mana: 0,
        },
      },
      condition: { type: 'target_critical' },
      cooldown: { remaining_ticks: 0 },
    };
  }
  if (attack_id === 'drain_magic_on_game_over') {
    return {
      id: attack_id,
      characteristics: {
        id: attack_id,
        name: 'Attack',
        target: { type: 'random_enemy' },
        action: { type: 'drain_mana', damage_mp: 1, drain_ratio: 1.0 },
        cost_data: {
          cooldown: 5,
          mana_percent: 0,
          mana: 0,
        },
      },
      condition: { type: 'target_game_over' },
      cooldown: { remaining_ticks: 0 },
    };
  }
  if (attack_id === 'drain_exp_on_game_over') {
    return {
      id: attack_id,
      characteristics: {
        id: attack_id,
        name: 'Attack',
        target: { type: 'random_enemy' },
        action: { type: 'drain_experience', damage_experience: 3, drain_ratio: 1.0 },
        cost_data: {
          cooldown: 5,
          mana_percent: 0,
          mana: 0,
        },
      },
      condition: { type: 'target_game_over' },
      cooldown: { remaining_ticks: 0 },
    };
  }
  assertTypeIsHandled(attack_id, 'attack id');
}

/**
 * @param { 'magic_cage_on_vulnerable'
 *   | 'magic_bindings_on_vulnerable'
 *    | 'rope_bindings_on_vulnerable'
 *   | 'petrify_on_critical'
 *   | 'slime_on_critical'
 *    | 'mirror_trap_on_critical'
 *   | 'magic_cage_on_critical'
 *   | 'magic_bindings_on_critical'
 *    | 'rope_bindings_on_critical'
 * } attack_id
 * @returns {import("../types").InGameEnemySkill}
 */
export function generateEffectEnemyAttack(attack_id) {
  if (attack_id === 'magic_cage_on_vulnerable') {
    return {
      id: attack_id,
      characteristics: {
        id: attack_id,
        name: 'Attack',
        target: { type: 'random_enemy' },
        action: { type: 'apply_effect', effect: 'magic_cage' },
        cost_data: {
          cooldown: 200,
          mana_percent: 0,
          mana: 0,
        },
      },
      condition: { type: 'target_vulnerable', vulnerability_ratio: 0.1 },
      cooldown: { remaining_ticks: 0 },
    };
  }
  if (attack_id === 'magic_bindings_on_vulnerable') {
    return {
      id: attack_id,
      characteristics: {
        id: attack_id,
        name: 'Attack',
        target: { type: 'random_enemy' },
        action: { type: 'apply_effect', effect: 'magic_bindings' },
        cost_data: {
          cooldown: 200,
          mana_percent: 0,
          mana: 0,
        },
      },
      condition: { type: 'target_vulnerable', vulnerability_ratio: 0.1 },
      cooldown: { remaining_ticks: 0 },
    };
  }
  if (attack_id === 'rope_bindings_on_vulnerable') {
    return {
      id: attack_id,
      characteristics: {
        id: attack_id,
        name: 'Attack',
        target: { type: 'random_enemy' },
        action: { type: 'apply_effect', effect: 'rope_bindings' },
        cost_data: {
          cooldown: 200,
          mana_percent: 0,
          mana: 0,
        },
      },
      condition: { type: 'target_vulnerable', vulnerability_ratio: 0.1 },
      cooldown: { remaining_ticks: 0 },
    };
  }
  if (attack_id === 'petrify_on_critical') {
    return {
      id: attack_id,
      characteristics: {
        id: attack_id,
        name: 'Attack',
        target: { type: 'random_enemy' },
        action: { type: 'apply_effect', effect: 'petrified' },
        cost_data: {
          cooldown: 50,
          mana_percent: 0,
          mana: 0,
        },
      },
      condition: { type: 'target_critical' },
      cooldown: { remaining_ticks: 0 },
    };
  }
  if (attack_id === 'mirror_trap_on_critical') {
    return {
      id: attack_id,
      characteristics: {
        id: attack_id,
        name: 'Attack',
        target: { type: 'random_enemy' },
        action: { type: 'apply_effect', effect: 'mirror_trap' },
        cost_data: {
          cooldown: 50,
          mana_percent: 0,
          mana: 0,
        },
      },
      condition: { type: 'target_critical' },
      cooldown: { remaining_ticks: 0 },
    };
  }
  if (attack_id === 'slime_on_critical') {
    return {
      id: attack_id,
      characteristics: {
        id: attack_id,
        name: 'Attack',
        target: { type: 'random_enemy' },
        action: { type: 'apply_effect', effect: 'slimed' },
        cost_data: {
          cooldown: 50,
          mana_percent: 0,
          mana: 0,
        },
      },
      condition: { type: 'target_critical' },
      cooldown: { remaining_ticks: 0 },
    };
  }
  if (attack_id === 'magic_cage_on_critical') {
    return {
      id: attack_id,
      characteristics: {
        id: attack_id,
        name: 'Attack',
        target: { type: 'random_enemy' },
        action: { type: 'apply_effect', effect: 'magic_cage' },
        cost_data: {
          cooldown: 50,
          mana_percent: 0,
          mana: 0,
        },
      },
      condition: { type: 'target_critical' },
      cooldown: { remaining_ticks: 0 },
    };
  }
  if (attack_id === 'magic_bindings_on_critical') {
    return {
      id: attack_id,
      characteristics: {
        id: attack_id,
        name: 'Attack',
        target: { type: 'random_enemy' },
        action: { type: 'apply_effect', effect: 'magic_bindings' },
        cost_data: {
          cooldown: 50,
          mana_percent: 0,
          mana: 0,
        },
      },
      condition: { type: 'target_critical' },
      cooldown: { remaining_ticks: 0 },
    };
  }
  if (attack_id === 'rope_bindings_on_critical') {
    return {
      id: attack_id,
      characteristics: {
        id: attack_id,
        name: 'Attack',
        target: { type: 'random_enemy' },
        action: { type: 'apply_effect', effect: 'rope_bindings' },
        cost_data: {
          cooldown: 50,
          mana_percent: 0,
          mana: 0,
        },
      },
      condition: { type: 'target_critical' },
      cooldown: { remaining_ticks: 0 },
    };
  }
  assertTypeIsHandled(attack_id, 'attack id');
}
