import { assertTypeIsHandled } from '../utils.mjs';
import { generateDamagingAttack, generateDrainingEnemyAttack, generateEffectEnemyAttack } from './enemy-attack.mjs';
import { getInGameStatisticsFromModifiers } from './stats-modifiers.mjs';

/**
 * @param { import("../types").EnemyKind } kind
 * @param {number} level
 * @returns { import("../types").PartialInGameStatistics }
 */
export function getEnemyPartialStats(kind, level) {
  if (kind === 'ghost') {
    return {
      max_hp: 10 + level * 5,
      max_mp: 10,
    };
  }
  if (kind === 'book') {
    return {
      max_hp: 20 + level * 20,
      max_mp: 10,
    };
  }
  if (kind === 'mage') {
    return {
      max_hp: 100 + level * 20,
      max_mp: 10,
    };
  }
  if (kind === 'rogue') {
    return {
      max_hp: 100 + level * 20,
      max_mp: 10,
    };
  }
  if (kind === 'slime') {
    return {
      max_hp: 10 + level * 5,
      max_mp: 10,
    };
  }
  if (kind === 'mirror') {
    return {
      max_hp: 10 + level * 5,
      max_mp: 10,
    };
  }
  if (kind === 'high_sorceress') {
    return {
      max_hp: 500 + level * 40,
      max_mp: 10,
    };
  }
  assertTypeIsHandled(kind, 'enemy kind');
}

/**
 * @param { import("../types").EnemyKind } kind
 * @param { number } level
 * @returns { import("../types").InGameEnemyData }
 */
export function generateEnemy(kind, level) {
  const stats = getInGameStatisticsFromModifiers(getEnemyPartialStats(kind, level), []);
  if (kind === 'ghost') {
    return {
      type: 'enemy',
      name: 'Ghost',
      kind: kind,
      animation: {
        current: 'none',
      },
      image: 'enemy1',
      level: level,
      exp_value: 10 + level * 10,
      base_stats: stats,
      current_stats: { hp: stats.max_hp, mp: stats.max_mp },
      skills: [
        generateDamagingAttack('damage_small'),
        generateEffectEnemyAttack('petrify_on_critical'),
        generateDrainingEnemyAttack('drain_magic_on_game_over'),
      ],
    };
  }
  if (kind === 'book') {
    return {
      type: 'enemy',
      name: 'Book',
      kind: kind,
      animation: {
        current: 'none',
      },
      image: 'enemy2',
      level: level,
      exp_value: 30 + level * 20,
      base_stats: stats,
      current_stats: { hp: stats.max_hp, mp: stats.max_mp },
      skills: [
        generateDamagingAttack('damage_medium'),
        generateDrainingEnemyAttack('drain_magic_small_on_critical'),
        generateEffectEnemyAttack('magic_cage_on_vulnerable'),
        generateEffectEnemyAttack('magic_cage_on_critical'),
        generateDrainingEnemyAttack('drain_magic_on_game_over'),
      ],
    };
  }
  if (kind === 'mage') {
    return {
      type: 'enemy',
      name: 'Sorceress',
      kind: kind,
      animation: {
        current: 'none',
      },
      image: 'enemy3',
      level: level,
      exp_value: 100 + level * 100,
      base_stats: stats,
      current_stats: { hp: stats.max_hp, mp: stats.max_mp },
      skills: [
        generateDamagingAttack('damage_large'),
        generateEffectEnemyAttack('magic_bindings_on_vulnerable'),
        generateEffectEnemyAttack('magic_bindings_on_critical'),
        generateDrainingEnemyAttack('drain_magic_large_on_critical'),
        generateDrainingEnemyAttack('drain_magic_on_game_over'),
      ],
    };
  }
  if (kind === 'rogue') {
    return {
      type: 'enemy',
      name: 'Rogue',
      kind: kind,
      animation: {
        current: 'none',
      },
      image: 'enemy4',
      level: level,
      exp_value: 100 + level * 100,
      base_stats: stats,
      current_stats: { hp: stats.max_hp, mp: stats.max_mp },
      skills: [
        generateDamagingAttack('damage_large'),
        generateEffectEnemyAttack('rope_bindings_on_vulnerable'),
        generateEffectEnemyAttack('rope_bindings_on_critical'),
        generateDrainingEnemyAttack('drain_magic_on_game_over'),
      ],
    };
  }
  if (kind === 'slime') {
    return {
      type: 'enemy',
      name: 'Slime',
      kind: kind,
      animation: {
        current: 'none',
      },
      image: 'enemy5',
      level: level,
      exp_value: 10 + level * 10,
      base_stats: stats,
      current_stats: { hp: stats.max_hp, mp: stats.max_mp },
      skills: [
        generateDamagingAttack('damage_small'),
        generateEffectEnemyAttack('slime_on_critical'),
        generateDrainingEnemyAttack('drain_magic_on_game_over'),
      ],
    };
  }
  if (kind === 'mirror') {
    return {
      type: 'enemy',
      name: 'Mirror',
      kind: kind,
      animation: {
        current: 'none',
      },
      image: 'enemy6',
      level: level,
      exp_value: 10 + level * 10,
      base_stats: stats,
      current_stats: { hp: stats.max_hp, mp: stats.max_mp },
      skills: [
        generateDamagingAttack('damage_small'),
        generateEffectEnemyAttack('mirror_trap_on_critical'),
        generateDrainingEnemyAttack('drain_magic_on_game_over'),
      ],
    };
  }
  if (kind === 'high_sorceress') {
    return {
      type: 'enemy',
      name: 'High Sorceress',
      kind: kind,
      animation: {
        current: 'none',
      },
      image: 'enemy7',
      level: level,
      exp_value: 1000 + level * 200,
      base_stats: stats,
      current_stats: { hp: stats.max_hp, mp: stats.max_mp },
      skills: [
        generateDamagingAttack('damage_large'),
        generateDamagingAttack('damage_very_large'),
        generateEffectEnemyAttack('petrify_on_critical'),
        generateEffectEnemyAttack('magic_cage_on_critical'),
        generateEffectEnemyAttack('magic_bindings_on_critical'),
        generateDrainingEnemyAttack('drain_magic_large_on_critical'),
        generateDrainingEnemyAttack('drain_exp_on_game_over'),
      ],
    };
  }
  assertTypeIsHandled(kind, 'enemy attack');
}
