import { generateEnemy } from './enemy-character.mjs';
import { getRandomInteger } from '../random.mjs';
import { enemy_spawn_data } from './enemy-spawn.mjs';

/**
 * @param {import("../types").DungeonGameData} data
 * @param {number} level
 * @param {"small_group" | "large_group" | "ambush"} spawn_mode
 * @returns { import("../types").InGameEnemyData[] }
 */
export function generateEnemiesForDungeon(data, level, spawn_mode) {
  /** @type { Array<{ quantity: number, weight: number }> } */
  const enemy_quantity_weights = [];
  /** @type { Array<{ kind: import('../types').EnemyKind, weight: number }> } */
  const enemy_kind_weights = [];

  for (const entry of enemy_spawn_data) {
    if (entry.type === 'small' && spawn_mode !== 'small_group') continue;
    if (entry.type === 'large' && spawn_mode !== 'large_group') continue;
    if (entry.type === 'ambush' && spawn_mode !== 'ambush') continue;
    if (level >= entry.min_level && level < entry.max_level) {
      if (entry.kind) {
        enemy_kind_weights.push({ kind: entry.kind.kind, weight: entry.kind.weight });
      }
      if (entry.quantity) {
        enemy_quantity_weights.push({ quantity: entry.quantity.quantity, weight: entry.quantity.weight });
      }
    }
  }

  let enemy_quantity = 1;
  const enemy_quantity_sum = enemy_quantity_weights.map((w) => w.weight).reduce((p, v) => p + v, 0);
  if (enemy_quantity_sum > 0) {
    const value = getRandomInteger(0, enemy_quantity_sum);
    let current = 0;
    for (const entry of enemy_quantity_weights) {
      current += entry.weight;
      if (current > value) {
        enemy_quantity = entry.quantity;
        break;
      }
    }
  }

  const enemies = [];
  for (let index = 0; index < enemy_quantity; ++index) {
    const enemy_kind_sum = enemy_kind_weights.map((w) => w.weight).reduce((p, v) => p + v, 0);
    if (enemy_kind_sum > 0) {
      const value = getRandomInteger(0, enemy_kind_sum);
      let current = 0;
      for (const entry of enemy_kind_weights) {
        current += entry.weight;
        if (current > value) {
          enemies.push(generateEnemy(entry.kind, level));
          break;
        }
      }
    }
  }

  return enemies;
}
