/** @type { Array<import("../types").StatusEffectsCharacteristics> } */
const status_effects_data = [
  {
    id: 'petrified',
    name: 'Petrification',
    modify_image: {
      type: 'per_stack',
      stacks: {
        1: 'character_body_petrified1_base',
        2: 'character_body_petrified2',
        3: 'character_body_petrified3',
      },
    },
    block_skills: {
      type: 'per_stack',
      stacks: {
        1: { type: 'none' },
        2: { type: 'all_except_exact', skills: ['resist_petrification'] },
        3: { type: 'all_except_exact', skills: ['resist_petrification'] },
      },
    },
    behavior: {
      type: 'staged_progression',
      resist_skill: 'resist_petrification',
      stages: [
        {
          stack: 1,
          tick_count: 200,
          next: { type: 'stack_upgrade', next_stack: 2 },
          next_messages: ['Body turning into stone!'],
        },
        {
          stack: 2,
          tick_count: 200,
          next: { type: 'stack_upgrade', next_stack: 3 },
          next_messages: ['Turning into stone!'],
        },
        {
          stack: 3,
          tick_count: 200,
          next: { type: 'change_status', next_status_effect_id: 'statue' },
          next_messages: ['Fully petrified!'],
        },
      ],
    },
  },
  {
    id: 'statue',
    name: 'Statue',
    modify_image: {
      type: 'static',
      image: 'character_body_statue',
    },
    block_skills: { type: 'static', blocks: { type: 'all' } },
    behavior: { type: 'game_over', new_role: 'statue' },
  },
  {
    id: 'mirror_trap',
    name: 'Mirror Trap',
    modify_image: {
      type: 'per_stack',
      stacks: {
        1: 'character_body_mirror1',
        2: 'character_body_mirror2',
        3: 'character_body_mirror3',
      },
    },
    block_skills: {
      type: 'per_stack',
      stacks: {
        1: { type: 'all_except_exact', skills: ['resist_mirror_trap'] },
        2: { type: 'all_except_exact', skills: ['resist_mirror_trap'] },
        3: { type: 'all_except_exact', skills: ['resist_mirror_trap'] },
      },
    },
    behavior: {
      type: 'staged_progression',
      resist_skill: 'resist_mirror_trap',
      stages: [
        {
          stack: 1,
          tick_count: 200,
          next: { type: 'stack_upgrade', next_stack: 2 },
          next_messages: ['Thrown into the mirror!'],
        },
        {
          stack: 2,
          tick_count: 200,
          next: { type: 'stack_upgrade', next_stack: 3 },
          next_messages: ['Mirror glass starts to solidify!'],
        },
        {
          stack: 3,
          tick_count: 200,
          next: { type: 'change_status', next_status_effect_id: 'mirrored' },
          next_messages: ['Mirror has solidified!'],
        },
      ],
    },
  },
  {
    id: 'mirrored',
    name: 'Mirrored',
    modify_image: {
      type: 'static',
      image: 'character_body_mirrored',
    },
    block_skills: { type: 'static', blocks: { type: 'all' } },
    behavior: { type: 'game_over', new_role: 'mirror' },
  },
  {
    id: 'magic_bindings',
    name: 'Magic Bindings',
    modify_image: {
      type: 'static',
      image: 'character_body_magic_bindings1',
    },
    block_skills: {
      type: 'static',
      blocks: { type: 'all_except_exact', skills: ['struggle_magic_bindings'] },
    },
    behavior: {
      type: 'staged_progression',
      resist_skill: 'struggle_magic_bindings',
      stages: [
        {
          stack: 1,
          tick_count: 200,
          next: { type: 'change_status', next_status_effect_id: 'magic_bindings_secured' },
          next_messages: ['Bindings secured!'],
        },
      ],
    },
  },
  {
    id: 'magic_bindings_secured',
    name: 'Bundled up',
    modify_image: {
      type: 'static',
      image: 'character_body_magic_bindings2',
    },
    block_skills: { type: 'static', blocks: { type: 'all' } },
    behavior: { type: 'game_over' },
  },
  {
    id: 'magic_cage',
    name: 'Magic Cage',
    modify_image: {
      type: 'per_stack',
      stacks: {
        1: 'character_body_magic_cage1',
        2: 'character_body_magic_cage2',
      },
    },
    block_skills: {
      type: 'static',
      blocks: { type: 'all_except_exact', skills: ['struggle_magic_cage'] },
    },
    behavior: {
      type: 'staged_progression',
      resist_skill: 'struggle_magic_cage',
      stages: [
        {
          stack: 1,
          tick_count: 200,
          next: { type: 'stack_upgrade', next_stack: 2 },
          next_messages: ['Cage rising up!'],
        },
        {
          stack: 2,
          tick_count: 200,
          next: { type: 'change_status', next_status_effect_id: 'magic_cage_secured' },
          next_messages: ['Cage locked!'],
        },
      ],
    },
  },
  {
    id: 'magic_cage_secured',
    name: 'Locked Away',
    modify_image: {
      type: 'static',
      image: 'character_body_magic_cage3',
    },
    block_skills: { type: 'static', blocks: { type: 'all' } },
    behavior: { type: 'game_over' },
  },
  {
    id: 'rope_bindings',
    name: 'Rope Bindings',
    modify_image: {
      type: 'static',
      image: 'character_body_rope_bindings1',
    },
    block_skills: {
      type: 'static',
      blocks: { type: 'all_except_exact', skills: ['struggle_rope_bindings'] },
    },
    behavior: {
      type: 'staged_progression',
      resist_skill: 'struggle_rope_bindings',
      stages: [
        {
          stack: 1,
          tick_count: 200,
          next: { type: 'change_status', next_status_effect_id: 'rope_bindings_secured' },
          next_messages: ['Bindings secured!'],
        },
      ],
    },
  },
  {
    id: 'rope_bindings_secured',
    name: 'Bundled up',
    modify_image: {
      type: 'static',
      image: 'character_body_rope_bindings2',
    },
    block_skills: { type: 'static', blocks: { type: 'all' } },
    behavior: { type: 'game_over' },
  },
  {
    id: 'slimed',
    name: 'Slimed!',
    modify_image: {
      type: 'static',
      image: 'character_body_slime1_base',
    },
    block_skills: {
      type: 'static',
      blocks: { type: 'all_except_exact', skills: ['struggle_slime'] },
    },
    behavior: {
      type: 'staged_progression',
      resist_skill: 'struggle_slime',
      stages: [
        {
          stack: 1,
          tick_count: 200,
          next: { type: 'change_status', next_status_effect_id: 'slimed_secured' },
          next_messages: ['Slime solidified!'],
        },
      ],
    },
  },
  {
    id: 'slimed_secured',
    name: 'Trapped in slime',
    modify_image: {
      type: 'static',
      image: 'character_body_slime2',
    },
    block_skills: { type: 'static', blocks: { type: 'all' } },
    behavior: { type: 'game_over' },
  },
];

/**
 * @param { import("../types").StatusEffectId } id
 * @returns { import("../types").StatusEffectsCharacteristics }
 */
export function getStatusEffectCharacteristics(id) {
  const data = status_effects_data.find((s) => s.id === id);
  if (!data) throw new Error(`Could not find characteristics of status effect ${id}`);
  return data;
}

/**
 * @param {import("../types").InGameStatusEffect} effect
 * @returns {string}
 */
export function getStatusEffectDescription(effect) {
  let text = `${effect.characteristics.name}`;
  if (effect.characteristics.behavior.type === 'game_over') {
    text += '\nPermanent (game over)';
    return text;
  }
  if (effect.current_stack > 1) text += `\nStage ${effect.current_stack}`;
  if (effect.lifetime.type === 'fixed') text += '\nPermanent';
  if (effect.lifetime.type === 'cooldown')
    text += `\n${Math.floor(effect.lifetime.remaining_ticks / 2) / 10}s remaining`;
  if (effect.lifetime.type === 'coolup') {
    text += `\n${Math.floor(effect.lifetime.ongoing_ticks / 2) / 10}`;
    text += ` / ${Math.floor(effect.lifetime.max_ticks / 2) / 10}`;
    text += `s before next stage`;
  }
  return text;
}
