/**
 * @param {import("../types").GameData} game_data
 * @returns {import("../types").StoredEquipmentData[]}
 */
export function getAllEquipmentsInTown(game_data) {
  const items = [];
  for (const item of game_data.town.unused_equipment) items.push(item);
  if (game_data.town.cleanup_equipment.active) items.push(game_data.town.cleanup_equipment.target);
  if (game_data.town.upgrade_equipment.active && game_data.town.upgrade_equipment.target)
    items.push(game_data.town.upgrade_equipment.target);
  for (const slot of game_data.town.expedition_slots) {
    if (slot.equipment.body) items.push(slot.equipment.body);
    if (slot.equipment.accessory) items.push(slot.equipment.accessory);
    if (slot.equipment.weapon) items.push(slot.equipment.weapon);
  }
  return items;
}

/**
 * @param {import("../types").GameData} game_data
 * @returns {import("../types").StoredEquipmentData[]}
 */
export function getAllEquipments(game_data) {
  const items = getAllEquipmentsInTown(game_data);
  for (const item of game_data.game_state.items_outside_town) items.push(item);
  for (const item of game_data.dungeon.rewards.gathered_equipments) items.push(item);
  return items;
}

/**
 * @param {import("../types").GameData} game_data
 * @returns {import("../types").StoredCharacterData[]}
 */
export function getAllCharactersInTown(game_data) {
  const characters = [];
  for (const character of game_data.town.available_team) characters.push(character);
  if (game_data.town.upgrade_equipment.active && game_data.town.upgrade_equipment.handler)
    characters.push(game_data.town.upgrade_equipment.handler);
  if (game_data.town.change_role.active) characters.push(game_data.town.change_role.target);
  for (const slot of game_data.town.expedition_slots) {
    if (slot.character) characters.push(slot.character);
  }
  return characters;
}

/**
 * @param {import("../types").GameData} game_data
 * @returns {import("../types").StoredCharacterData[]}
 */
export function getAllLiveCharacters(game_data) {
  const characters = getAllCharactersInTown(game_data);
  for (const character of game_data.game_state.characters_outside_town) characters.push(character);
  for (const character of game_data.dungeon.team) characters.push(character.base);
  for (const character of game_data.dungeon.rewards.saved_characters) characters.push(character);
  return characters;
}

/**
 * @param {import("../types").GameData} game_data
 * @returns {import("../types").StoredCharacterData[]}
 */
export function getAllLiveTransmutedAndDustedCharacters(game_data) {
  const equipments = getAllEquipments(game_data);
  const characters = getAllLiveCharacters(game_data);
  for (const character of game_data.game_state.dusted_characters) characters.push(character);
  for (const equipment of equipments) {
    if (equipment.character) characters.push(equipment.character.data);
  }
  return characters;
}
