import { getRelationsFor } from '../data/backstory.mjs';

/**
 * @template T
 * @template {keyof T} K
 * @typedef {T extends any ? Omit<T, K> : never} DistributiveOmit
 */

/**
 * @param {import("../types").GameData} game_data
 * @param {import("../types").StoredCharacterData} character
 * @param {DistributiveOmit<import("../types").CharacterBackstoryTownEvent, "cycle">} story
 */
export function addTownBackstoryEvent(game_data, character, story) {
  const entry = (() => {
    const last_backstory = character.story.events[character.story.events.length - 1];
    if (last_backstory.location === 'town') {
      return last_backstory;
    }
    /** @type {import("../types").CharacterBackstoryEvent} */
    const event_group = {
      location: 'town',
      cycle_start: game_data.game_state.current_cycle,
      content: [],
    };
    character.story.events.push(event_group);
    return event_group;
  })();
  entry.content.push({ ...story, cycle: game_data.game_state.current_cycle });
  if (story.type === 'friendship_start') {
    const relation = getRelationsFor(character, story.friend.character);
    relation.status.is_friend = true;
    relation.interactions.friendship_interactions++;
  } else if (story.type === 'friendship_event') {
    const relation = getRelationsFor(character, story.friend.character);
    relation.interactions.friendship_interactions++;
  } else if (story.type === 'friendship_end') {
    const relation = getRelationsFor(character, story.other.character);
    relation.status.is_friend = false;
  } else if (story.type === 'rivalry_start_bullies') {
    const relation = getRelationsFor(character, story.rival.character);
    relation.status.is_rival_target = true;
    relation.interactions.rivalry_bullies_interactions++;
  } else if (story.type === 'rivalry_start_is_bullied') {
    const relation = getRelationsFor(character, story.rival.character);
    relation.status.is_rival_bully = true;
    relation.interactions.rivalry_is_bullied_interactions++;
  } else if (story.type === 'rivalry_event_bullies') {
    const relation = getRelationsFor(character, story.rival.character);
    relation.interactions.rivalry_bullies_interactions++;
  } else if (story.type === 'rivalry_event_is_bullied') {
    const relation = getRelationsFor(character, story.rival.character);
    relation.interactions.rivalry_is_bullied_interactions++;
  } else if (story.type === 'rivalry_end') {
    const relation = getRelationsFor(character, story.rival.character);
    relation.status.is_rival_bully = false;
    relation.status.is_rival_target = false;
  } else if (story.type === 'romance_start') {
    const relation = getRelationsFor(character, story.girlfriend.character);
    relation.status.is_partner = true;
    relation.status.is_rival_bully = false;
    relation.interactions.romance_interactions++;
  } else if (story.type === 'romance_event') {
    const relation = getRelationsFor(character, story.girlfriend.character);
    relation.interactions.romance_interactions++;
  }
}
