import { generateEquipmentStatisticsForModifier, recomputeEquipmentTargetCycle } from '../data/equipment.mjs';
import { changeStoredCharacterRoleTo } from '../data/player-character.mjs';
import { addTownBackstoryEvent } from './add-backstory-town.mjs';
import { addTransmutedBackstoryEvent } from './add-backstory-transmuted.mjs';
import { getEventCharacterInfo } from './add-backstory.mjs';

/**
 * @param {import("../types").GameData} game_data
 */
export function handleTownUpgradeEvents(game_data) {
  if (game_data.town.change_role.active) {
    if (game_data.town.change_role.retarget_end_cycle <= game_data.game_state.current_cycle) {
      changeStoredCharacterRoleTo(game_data.town.change_role.target, game_data.town.change_role.target_role);
      addTownBackstoryEvent(game_data, game_data.town.change_role.target, {
        type: 'class_changed_event',
        new_role: game_data.town.change_role.target_role,
      });
      game_data.town.available_team.push(game_data.town.change_role.target);
      game_data.town.change_role = { active: false };
    }
  }

  if (
    game_data.town.upgrade_equipment.active &&
    game_data.town.upgrade_equipment.handler &&
    game_data.town.upgrade_equipment.target &&
    game_data.town.upgrade_equipment.retarget_end_cycle !== 0
  ) {
    if (game_data.town.upgrade_equipment.retarget_end_cycle <= game_data.game_state.current_cycle) {
      if (game_data.town.use_dust_for_upgrade) {
        let level_diff =
          game_data.town.upgrade_equipment.handler.level - game_data.town.upgrade_equipment.target.level - 1;
        const used_dust = Math.max(level_diff, game_data.town.available_dust);
        if (used_dust > 0) game_data.town.available_dust -= used_dust;
      }
      game_data.town.upgrade_equipment.target.level += 1;
      game_data.town.upgrade_equipment.target.stats = generateEquipmentStatisticsForModifier(
        game_data.town.upgrade_equipment.target.stats.type,
        game_data.town.upgrade_equipment.target.level,
      );
      if (game_data.town.upgrade_equipment.target.character) {
        const character = game_data.town.upgrade_equipment.target.character.data;
        addTransmutedBackstoryEvent(game_data, character, {
          type: 'team_upgrade_event',
          equipment: {
            name: game_data.town.upgrade_equipment.target.name,
            level: game_data.town.upgrade_equipment.target.level,
          },
          teammate: getEventCharacterInfo(character, game_data.town.upgrade_equipment.handler.id),
        });
      }
      addTownBackstoryEvent(game_data, game_data.town.upgrade_equipment.handler, {
        type: 'upgrade_event',
        item: {
          name: game_data.town.upgrade_equipment.target.name,
          level: game_data.town.upgrade_equipment.target.level,
        },
      });
      recomputeEquipmentTargetCycle(game_data, game_data.town);
    }
  }
}
