import { getRandomInArray } from '../random.mjs';
import { createCapturedBroughtInDungeonBackstory } from './add-backstory-captured.mjs';
import { addTownBackstoryEvent } from './add-backstory-town.mjs';
import { getEventCharacterInfo } from './add-backstory.mjs';

/**
 * @param {import("../types").GameData} game_data The current game
 */
export function forceFullGameOver(game_data) {
  const state = game_data.game_state;
  const town = game_data.town;

  const captured_characters = [];
  for (const team of town.available_team) {
    state.characters_outside_town.push(team);
    captured_characters.push(team);
  }
  town.available_team = [];
  for (const slot of town.expedition_slots) {
    if (slot.character) {
      state.characters_outside_town.push(slot.character);
      captured_characters.push(slot.character);
    }
    if (slot.equipment.body) state.items_outside_town.push(slot.equipment.body);
    if (slot.equipment.accessory) state.items_outside_town.push(slot.equipment.accessory);
    if (slot.equipment.weapon) state.items_outside_town.push(slot.equipment.weapon);
    slot.character = undefined;
    slot.equipment.body = undefined;
    slot.equipment.accessory = undefined;
    slot.equipment.weapon = undefined;
  }
  if (town.upgrade_equipment.active) {
    if (town.upgrade_equipment.handler) {
      state.characters_outside_town.push(town.upgrade_equipment.handler);
      captured_characters.push(town.upgrade_equipment.handler);
      town.upgrade_equipment.handler = null;
    }
    if (town.upgrade_equipment.target) {
      state.items_outside_town.push(town.upgrade_equipment.target);
      town.upgrade_equipment.target = null;
    }
    town.upgrade_equipment = { active: false };
  }
  if (town.change_role.active) {
    state.characters_outside_town.push(town.change_role.target);
    captured_characters.push(town.change_role.target);
    town.change_role = { active: false };
  }
  for (const equipment of town.unused_equipment) state.items_outside_town.push(equipment);
  town.unused_equipment = [];

  for (const character of captured_characters) {
    const other_captured_characters = captured_characters.filter((c) => c !== character);
    if (other_captured_characters.length === 0) {
      addTownBackstoryEvent(game_data, character, { type: 'alone_capture_town_corruption' });
      createCapturedBroughtInDungeonBackstory(game_data, character, other_captured_characters);
    } else {
      addTownBackstoryEvent(game_data, character, {
        type: 'group_capture_town_corruption',
        capturedGroup: other_captured_characters.map((c) => getEventCharacterInfo(character, c.id)),
        kind: getRandomInArray(['ambushed', 'fought_back', 'overpowered']),
      });
      createCapturedBroughtInDungeonBackstory(game_data, character, other_captured_characters);
    }
  }

  if (state.game_over_cooldown === -1) state.game_over_cooldown = 160;
}
