import { generateInGameCharacterFromData } from '../data/player-character.mjs';
import { addExplorationBackstoryEvent } from './add-backstory-exploration.mjs';
import { addTransmutedBackstoryEvent } from './add-backstory-transmuted.mjs';
import { getEventCharacterInfo } from './add-backstory.mjs';

/**
 * @param {import("../types").GameData} game_data
 */
export function leaveTownForDungeon(game_data) {
  const data = game_data.town;
  const dungeon = game_data.dungeon;
  dungeon.enemy = [];
  dungeon.level.is_round_active = false;
  dungeon.level.exploration_cycles = 0;
  dungeon.spawn.ambush_cooldown = -1;
  dungeon.rewards.last_reward_message = '';
  dungeon.rewards.current_reward_chance = 0;
  dungeon.team = data.expedition_slots
    .map((slot) => {
      if (slot.character) return generateInGameCharacterFromData(slot.character, slot.equipment);
      return null;
    })
    .filter((character) => !!character);
  dungeon.enemy = [];
  for (const slot of data.expedition_slots) {
    if (!slot.character) {
      if (slot.equipment.body) data.unused_equipment.push(slot.equipment.body);
      if (slot.equipment.accessory) data.unused_equipment.push(slot.equipment.accessory);
      if (slot.equipment.weapon) data.unused_equipment.push(slot.equipment.weapon);
    }
    slot.character = undefined;
    slot.equipment = {};
  }
  for (const member of dungeon.team) {
    const equipments = [member.equipment.accessory, member.equipment.body, member.equipment.weapon].filter((e) => !!e);
    for (const equipment of equipments) {
      if (!equipment.character) continue;
      const character = equipment.character.data;
      const otherTeamMembers = equipments
        .filter((e) => e !== equipment)
        .map((e) => e.character)
        .filter((e) => !!e)
        .map((e) => e.data);
      if (otherTeamMembers.length > 0) {
        addTransmutedBackstoryEvent(game_data, character, {
          type: 'team_group_use_event',
          capturedGroup: otherTeamMembers.map((e) => getEventCharacterInfo(character, e.id)),
          teammate: getEventCharacterInfo(character, member.base.id),
        });
      } else {
        addTransmutedBackstoryEvent(game_data, character, {
          type: 'team_use_event',
          teammate: getEventCharacterInfo(character, member.base.id),
        });
      }
    }
  }

  if (dungeon.team.length === 1) {
    const sole_teammate = dungeon.team[0];
    addExplorationBackstoryEvent(game_data, sole_teammate.base, { type: 'alone_start_exploration' });
  } else {
    for (const teammate of dungeon.team) {
      addExplorationBackstoryEvent(game_data, teammate.base, {
        type: 'team_start_exploration',
        group: dungeon.team.filter((t) => t !== teammate).map((t) => getEventCharacterInfo(teammate.base, t.base.id)),
      });
    }
  }
  game_data.game_state.screen = 'dungeon';
}
