import {
  generateEquipmentStatisticsForType,
  generateStoredAccessoryEquipment,
  generateStoredBodyEquipment,
  generateStoredWeaponEquipment,
} from './data/equipment.mjs';
import { generateDefaultAvailableTeam } from './data/player-character.mjs';

export const EXPECTED_VERSION = '823';

/** @returns {import("./types").GameData} */
export function getGameData() {
  return {
    version: EXPECTED_VERSION,
    game_state: {
      screen: 'town',
      current_cycle: 0,
      current_corruption_ratio: 0,
      characters_outside_town: [],
      dusted_characters: [],
      items_outside_town: [
        generateStoredBodyEquipment('cloth', generateEquipmentStatisticsForType('cloth', 1), 1),
        generateStoredBodyEquipment('leather', generateEquipmentStatisticsForType('leather', 1), 1),
        generateStoredBodyEquipment('plated', generateEquipmentStatisticsForType('plated', 1), 1),
        generateStoredAccessoryEquipment('necklace', generateEquipmentStatisticsForType('necklace', 1), 1),
      ],
      dungeon_knowledge: {
        interrogations_completed: [],
        girlfriends_known: [],
        transmutation_states: [],
        forced_drain_into: [],
      },
      game_over_cooldown: -1,
      game_over_scroll: { is_elevator_selected: false, scroll_offset: 0 },
      game_over_backstory: { is_elevator_selected: false, scroll_offset: 0 },
    },
    pause: {
      is_paused: false,
      pause_button_held: false,
      effects_volume: { is_elevator_selected: false, scroll_offset: 1.0 },
      music_volume: { is_elevator_selected: false, scroll_offset: 0.0 },
    },
    town: {
      screen: 'town',
      background_image: 'background_forest',
      unused_equipment: [
        generateStoredWeaponEquipment('staff', generateEquipmentStatisticsForType('staff', 1), 1),
        generateStoredWeaponEquipment('staff', generateEquipmentStatisticsForType('staff', 1), 1),
        generateStoredWeaponEquipment('sword', generateEquipmentStatisticsForType('sword', 1), 1),
        generateStoredWeaponEquipment('sword', generateEquipmentStatisticsForType('sword', 1), 1),
        generateStoredWeaponEquipment('sword', generateEquipmentStatisticsForType('sword', 1), 1),
      ],
      available_team: generateDefaultAvailableTeam(),
      available_dust: 0,
      expedition_slots: [{ equipment: {} }, { equipment: {} }, { equipment: {} }, { equipment: {} }],
      change_role: { active: false },
      use_dust_for_upgrade: false,
      upgrade_equipment: { active: false },
      cleanup_equipment: { active: false },
      backstory: { scroll_offset: 1.0, is_elevator_selected: false },
      backstory_characters_controller: { scroll_offset: 0.0, is_elevator_selected: false },
      town_characters_controller: { scroll_offset: 0.0, is_elevator_selected: false },
      role_characters_controller: { scroll_offset: 0.0, is_elevator_selected: false },
      upgrade_characters_controller: { scroll_offset: 0.0, is_elevator_selected: false },
      expedition_characters_controller: { scroll_offset: 0.0, is_elevator_selected: false },
      town_equipment_controller: { scroll_offset: 0.0, is_elevator_selected: false },
      upgrade_equipment_controller: { scroll_offset: 0.0, is_elevator_selected: false },
      cleanup_equipment_controller: { scroll_offset: 0.0, is_elevator_selected: false },
      expedition_equipment_controller: { scroll_offset: 0.0, is_elevator_selected: false },
    },
    dungeon: {
      level: {
        background_image: 'background_dungeon',
        exploration_cycles: 0,
        is_round_active: false,
      },
      spawn: {
        ambush_cooldown: -1,
        ambush_time: 30 * 5,
      },
      rewards: {
        current_reward_chance: 0,
        last_reward_message: '',
        saved_characters: [],
        gathered_equipments: [],
      },
      team: [],
      enemy: [],
    },
    particles: [],
    popup: { state: 'none' },
  };
}
