/**
 * From https://stackoverflow.com/a/47593316
 * @param {number} a
 * @param {number} b
 * @param {number} c
 * @param {number} d
 * @returns {() => number}
 */
function sfc32(a, b, c, d) {
  return function () {
    a >>>= 0;
    b >>>= 0;
    c >>>= 0;
    d >>>= 0;
    let t = (a + b) | 0;
    a = b ^ (b >>> 9);
    b = (c + (c << 3)) | 0;
    c = (c << 21) | (c >>> 11);
    d = (d + 1) | 0;
    t = (t + d) | 0;
    c = (c + t) | 0;
    return (t >>> 0) / 4294967296;
  };
}
const seed = 1000 + Math.floor(Math.random() * 10000);
const random = sfc32(0x9e3779b9, 0x243f6a88, 0xb7e15162, seed ^ 0xdeadbeef);

/**
 * Gets a random number
 * @returns {number} A random number between 0 and 1
 */
export function getRandomNumber() {
  return random();
}

/**
 * @param {number} start_included
 * @param {number} end_not_included
 * @returns {number}
 */
export function getRandomInteger(start_included, end_not_included) {
  return Math.floor(start_included + getRandomNumber() * (end_not_included - start_included));
}

/**
 * @param {number} true_probability The probably that true gets picked, between 0 (never happening) and 1 (always happening)
 * @returns {boolean}
 */
export function binaryChoiceResult(true_probability) {
  if (true_probability >= 1) return true;
  if (true_probability <= 0) return false;
  return getRandomNumber() < true_probability;
}

/**
 * @template T
 * @param {readonly T[]} array
 * @returns {T}
 */
export function getRandomInArray(array) {
  const index = getRandomInteger(0, array.length);
  return array[index];
}

/**
 * @template T
 * @param {ReadonlyArray<{ data: T, weight: number }>} array
 * @returns {T}
 */
export function getRandomInWeightedArray(array) {
  const sum = array.reduce((p, v) => p + v.weight, 0);
  const random = getRandomNumber() * sum;
  let running = 0;
  let selected = undefined;
  for (const choice of array) {
    running += choice.weight;
    if (running >= random) {
      selected = choice;
      break;
    }
  }
  if (!selected) return array[0]?.data;
  return selected.data;
}

/**
 * Generate a new UUID
 * @returns {string} A new UUID
 */
export function generateUUID() {
  return 'xxxxxxxx-xxxx-4xxx-yxxx-xxxxxxxxxxxx'.replace(/[xy]/g, function (c) {
    const raw_value = getRandomInteger(0, 16);
    if (c == 'x') return raw_value.toString(16);
    const clean_value = (raw_value & 0x3) | 0x8;
    return clean_value.toString(16);
  });
}
