import { getEnemyRespawnCooldownSystem } from './enemy-respawn.mjs';
import { getEnemyDungeonSystem } from './enemy.mjs';
import { getPlayerDungeonSystem } from './player.mjs';

/**
 * @param {{ [key: string]: ImageBitmap }} images The loaded images to use by the renderer
 */
export function getDungeonSystem(images) {
  const playerSystem = getPlayerDungeonSystem(images);
  const enemySystem = getEnemyDungeonSystem(images);
  const enemyRespawnCooldownSystem = getEnemyRespawnCooldownSystem(images);
  /**
   * @param {import("../../engine.mjs").Game} game The current game
   * @param {number} elapsedTicks The elapsed ticks since the last update
   */
  return function dungeonSystem(game, elapsedTicks) {
    if (game.game_data.game_state.screen !== 'dungeon') return;
    const data = game.game_data.dungeon;
    game.renderer.renderRawImage(images[data.level.background_image], 0, 0, 800, 600);
    handleGameOver(game, data);

    game.audio.stop_loop('town_bass');
    game.audio.stop_loop('town_twinkles');
    game.audio.stop_loop('town_lead_prep');
    game.audio.stop_loop('town_lead_wait');
    game.audio.start_loop('combat_drum');
    game.audio.start_loop('combat_bass');
    game.audio.start_loop('combat_kicks');
    game.audio.start_loop('combat_lead');
    game.audio.loop_enable('combat_drum');

    for (const character of data.team) {
      playerSystem.handleCharacterLogic(game, data, character, elapsedTicks);
      playerSystem.displayCharacter(game, data, character, elapsedTicks);
    }

    if (data.enemy.length) {
      game.audio.loop_disable('combat_lead');
      game.audio.loop_enable('combat_bass');
      game.audio.loop_enable('combat_kicks');
      for (const character of data.enemy) {
        enemySystem.handleEnemyLogic(game, data, character, elapsedTicks);
        enemySystem.displayEnemy(game, data, character, elapsedTicks);
        enemySystem.handleEnemyDeath(game, data, character, elapsedTicks);
      }
    } else {
      game.audio.loop_enable('combat_lead');
      game.audio.loop_disable('combat_bass');
      game.audio.loop_disable('combat_kicks');
      enemyRespawnCooldownSystem.handleEnemyRespawnCooldown(game, data, elapsedTicks);
    }
  };

  /**
   * @param { import("../../engine.mjs").Game } game The current game
   * @param { import('../../types').DungeonGameData } data
   */
  function handleGameOver(game, data) {
    if (data.team.some((t) => !t.status_effects.some((s) => s.characteristics.behavior.type === 'game_over'))) return;
    if (game.game_data.game_state.game_over_cooldown >= 0) return;
    game.game_data.game_state.game_over_cooldown = 160;
  }
}
