import { getCharacterPosition, getEnemyPosition } from './utils.mjs';

/**
 * @param {import("../../engine.mjs").Game} game
 * @param {{ x: number, y: number }} position
 * @param {string} text
 * @param {{ r: number, g: number, b: number }} color
 * @param { "left" | "centered" | "right" } alignment
 */
export function generateRisingParticleAt(game, position, text, color, alignment) {
  let start_y = position.y;
  while (game.game_data.particles.some((p) => p.x === position.x && p.y + 15 > start_y)) {
    start_y += 15;
  }
  let start_x = position.x;
  game.context.fillStyle = 'black';
  game.context.font = `15px sans-serif`;
  const text_size = game.context.measureText(text);
  if (alignment === 'left') {
    start_x = position.x;
  } else if (alignment === 'centered') {
    start_x = position.x - text_size.width / 2;
  } else if (alignment === 'right') {
    start_x = position.x - text_size.width;
  }
  game.game_data.particles.push({
    x: start_x,
    y: start_y,
    dx: 0,
    dy: -2,
    lifetime: 100,
    alpha_lifetime: 20,
    color: color,
    text: text,
  });
}

/**
 * @param {import("../../engine.mjs").Game} game
 * @param {import("../../types").InGameCharacterData} character
 * @param {string} text
 * @param {{ r: number, g: number, b: number }} color
 */
export function generatePlayerParticle(game, character, text, color) {
  const character_position = getCharacterPosition(game, character);
  generateRisingParticleAt(
    game,
    { x: character_position.x + 50, y: character_position.y + 175 },
    text,
    color,
    'centered',
  );
}

/**
 * @param {import("../../engine.mjs").Game} game
 * @param {import("../../types").InGameEnemyData} enemy
 * @param {string} text
 * @param {{ r: number, g: number, b: number }} color
 */
export function generateEnemyParticle(game, enemy, text, color) {
  const base_position = getEnemyPosition(game, enemy);
  generateRisingParticleAt(game, { x: base_position.x + 60, y: base_position.y + 150 }, text, color, 'centered');
}

/**
 * @param {import("../../engine.mjs").Game} game
 * @param {import('../../types').InGameCharacterData} character
 * @param {import('../../types').InGameEnemyData} enemy
 * @param {string} text
 * @param {{ r: number, g: number, b: number }} color
 */
export function generatePlayerToEnemyDrainParticle(game, character, enemy, text, color) {
  const character_position = getCharacterPosition(game, character);
  const enemy_position = getEnemyPosition(game, enemy);
  const x_travel_over_lifetime = enemy_position.x - character_position.x;
  const y_travel_over_lifetime = enemy_position.y - character_position.y;
  const lifetime = 100;
  game.game_data.particles.push({
    x: character_position.x + 20,
    y: character_position.y + 150,
    dx: x_travel_over_lifetime / lifetime,
    dy: y_travel_over_lifetime / lifetime,
    lifetime: lifetime,
    alpha_lifetime: 20,
    color: color,
    text: text,
  });
}
