/**
 *
 * @param {{ x: number, y: number, width: number, height: number }} area
 * @param {{ x: number, y: number }} point
 */
export function pointIsInArea(area, point) {
  if (area.x > point.x) return false;
  if (area.x + area.width < point.x) return false;
  if (area.y > point.y) return false;
  if (area.y + area.height < point.y) return false;
  return true;
}

/**
 * @param {import("../../engine.mjs").Game} game The current game
 * @param {import("../../types").InGameCharacterData} character
 * @returns {{ x: number, y: number }}
 */
export function getCharacterPosition(game, character) {
  const position = game.game_data.dungeon.team.indexOf(character);
  return {
    x: position * 125,
    y: 100,
  };
}

/**
 * @param {import("../../engine.mjs").Game} game The current game
 * @param {import("../../types").InGameEnemyData} enemy
 * @returns {{ x: number, y: number }}
 */
export function getEnemyPosition(game, enemy) {
  const position = game.game_data.dungeon.enemy.indexOf(enemy);
  return {
    x: 500 + position * 75,
    y: position % 2 ? 400 : 100,
  };
}

/**
 *
 * @param {import("../../engine.mjs").Game} game
 * @param {{ x: number, y: number, width: number, height: number }} size
 * @param {number} percent
 * @param {string} fill
 * @param {string} back
 * @param {string} text
 */
export function drawBar(game, size, percent, fill, back, text) {
  game.context.fillStyle = back;
  game.context.fillRect(size.x, size.y, size.width, size.height);
  game.context.fillStyle = fill;
  game.context.fillRect(size.x + 2, size.y + 2, (size.width - 4) * percent, size.height - 4);
  game.context.fillStyle = 'black';
  game.context.font = `${size.height - 2}px sans-serif`;
  const text_width = game.context.measureText(text);
  game.context.fillText(text, size.x + size.width / 2 - text_width.width / 2, size.y + size.height - 3);
}

/**
 * @param {import("../../engine.mjs").Game} game The current game
 * @param {number} x
 * @param {number} y
 * @param {number} radius
 * @param {number} remaining_ticks
 * @param {number} total_ticks
 */
export function drawCooldownCircle(game, x, y, radius, remaining_ticks, total_ticks) {
  const percent = 1 - remaining_ticks / total_ticks;
  game.context.fillStyle = '#333';
  game.context.beginPath();
  game.context.arc(x, y, radius, 0, 2 * Math.PI);
  game.context.fill();
  game.context.fillStyle = '#555';
  game.context.beginPath();
  game.context.moveTo(x, y);
  game.context.lineTo(x - radius, y);
  game.context.arc(x, y, radius, 0, 2 * Math.PI * percent);
  game.context.lineTo(x, y);
  game.context.fill();
  const time_text =
    remaining_ticks / 20 > 1 ? `${Math.floor(remaining_ticks / 20)}s` : `${Math.floor(remaining_ticks / 2) / 10}s`;
  game.context.fillStyle = 'white';
  game.context.font = `${Math.ceil(radius * 0.8)}px sans-serif`;
  const time_text_size = game.context.measureText(time_text);
  game.context.fillText(time_text, x - time_text_size.width / 2, y + radius / 3);
}

/**
 * @param {import("../../engine.mjs").Game} game The current game
 * @param {number} x
 * @param {number} y
 * @param {number} radius
 * @param {string} message
 * @param {string} color
 * @param {string} message_color
 */
export function drawIndicatorCircle(game, x, y, radius, message, color, message_color) {
  game.context.fillStyle = color;
  game.context.beginPath();
  game.context.arc(x, y, radius, 0, 2 * Math.PI);
  game.context.fill();
  game.context.fillStyle = message_color;
  game.context.font = `${Math.ceil(radius * 0.8)}px sans-serif`;
  const time_text_size = game.context.measureText(message);
  game.context.fillText(message, x - time_text_size.width / 2, y + radius / 3);
}
