/**
 * @typedef { import("../engine.mjs").Game } Game
 * @typedef { import("../types").GameData } GameData
 */

export function getParticlesSystem() {
  /**
   * @param {Game} game The current game
   * @param {number} elapsedTicks The elapsed ticks since the last update
   */
  return function getParticlesSystem(game, elapsedTicks) {
    handleParticles(game, game.game_data, elapsedTicks);
  };

  /**
   * @param {Game} game The current game
   * @param {GameData} data
   * @param {number} elapsedTicks The elapsed ticks since the last update
   */
  function handleParticles(game, data, elapsedTicks) {
    for (const particle of data.particles) {
      particle.lifetime -= elapsedTicks;
      particle.x += particle.dx;
      particle.y += particle.dy;
      const alpha = Math.min(1.0, particle.lifetime / particle.alpha_lifetime);
      game.context.font = '15px sans-serif';
      game.context.fillStyle = `rgba(${particle.color.r}, ${particle.color.g}, ${particle.color.b}, ${alpha})`;
      game.context.fillText(particle.text, particle.x, particle.y);
    }
    data.particles = data.particles.filter((p) => p.lifetime > 0);
  }
}
