import { renderButton } from '../components/button.mjs';
import { showHorizontalSlider } from '../components/slider-horizontal.mjs';

export function getPauseSystem() {
  /**
   * @param {import("../engine.mjs").Game} game The current game
   * @param {number} elapsedTicks The elapsed ticks since the last update
   */
  return function pauseSystemHandler(game, elapsedTicks) {
    if (!game.game_data.pause.is_paused) {
      if (game.game_data.pause.pause_button_held) {
        if (!game.keyboard.state.back) game.game_data.pause.pause_button_held = false;
      } else {
        if (game.keyboard.state.back) {
          game.game_data.pause.is_paused = true;
          game.game_data.pause.pause_button_held = true;
          return;
        }
      }
      return;
    }
    if (game.game_data.pause.pause_button_held) {
      if (!game.keyboard.state.back) game.game_data.pause.pause_button_held = false;
    } else {
      if (game.keyboard.state.back) {
        game.game_data.pause.is_paused = false;
        game.game_data.pause.pause_button_held = true;
        return;
      }
    }

    game.context.fillStyle = 'rgba(0, 0, 0, 0.8)';
    game.context.fillRect(0, 0, 800, 600);

    game.context.fillStyle = '#AAA';
    game.context.fillRect(300, 180, 100, 20);
    game.context.font = '18px sans-serif';
    game.context.fillStyle = 'black';
    game.context.fillText('Music', 310, 195);
    showHorizontalSlider(
      game,
      true,
      { x: 300, y: 200, width: 200, height: 20, border_size: 2, elevator_width: 20 },
      { background: 'transparent', border: '#888', elevator: '#AAA', hovered_elevator: '#CCC' },
      game.game_data.pause.music_volume,
    );
    game.audio.set_volume('music', game.game_data.pause.music_volume.scroll_offset);

    game.context.fillStyle = '#AAA';
    game.context.fillRect(300, 230, 100, 20);
    game.context.font = '18px sans-serif';
    game.context.fillStyle = 'black';
    game.context.fillText('Effects', 310, 245);
    showHorizontalSlider(
      game,
      true,
      { x: 300, y: 250, width: 200, height: 20, border_size: 2, elevator_width: 20 },
      { background: 'transparent', border: '#888', elevator: '#AAA', hovered_elevator: '#CCC' },
      game.game_data.pause.effects_volume,
    );
    game.audio.set_volume('effect', game.game_data.pause.effects_volume.scroll_offset);

    const continue_button = renderButton(game, true, {
      x: 300,
      y: 350,
      background: { base: '#AAA', hover: '#CCC' },
      content: { type: 'fixed_text', text: 'Continue', font_size: 30, height: 75, width: 200 },
    });
    if (continue_button.is_hovered && game.mouse.primary_button_clicked_frame) {
      game.sound_queue.push({ sound: 'click' });
      game.game_data.pause.is_paused = false;
    }

    const quit_button = renderButton(game, true, {
      x: 300,
      y: 450,
      background: { base: '#AAA', hover: '#CCC' },
      content: { type: 'fixed_text', text: 'Quit', font_size: 30, height: 75, width: 200 },
    });
    if (quit_button.is_hovered && game.mouse.primary_button_clicked_frame) {
      game.sound_queue.push({ sound: 'click' });
      game.shouldQuit = true;
    }
  };
}
