import { renderButton } from '../../components/button.mjs';
import { showPopupOnMousePosition } from '../../components/popup.mjs';
import { recomputeEquipmentTargetCycle } from '../../data/equipment.mjs';
import { displayAvailableEquipments, display_equipment } from './utils.mjs';

/**
 * @param {{ [key: string]: ImageBitmap }} images The loaded images to use by the renderer
 */
export function getEquipmentCleanupScreenSystem(images) {
  return {
    /**
     * @param {import("../../engine.mjs").Game} game The current game
     * @param {import('../../types').TownGameData} data The current game
     */
    displayEquipmentCleanupScreen: function (game, data) {
      const back_to_town = renderButton(game, !game.game_data.pause.is_paused, {
        x: 50,
        y: 30,
        background: { base: '#AAA', hover: '#CCC' },
        content: { type: 'text', text: 'Back', font_size: 20, padding: { x: 5, y: 5 } },
      });
      if (back_to_town.is_hovered && !game.game_data.pause.is_paused && game.mouse.primary_button_clicked_frame) {
        game.sound_queue.push({ sound: 'click' });
        data.screen = 'town';
      }

      displayUnusedItems(game, data);

      displayEquipmentCleanup(game, data);

      display_equipment.dust.detailed(game, images, { x: 670, y: 470 });
    },
  };

  /**
   * @param {import("../../engine.mjs").Game} game The current game
   * @param {import('../../types').TownGameData} data The current game
   */
  function displayUnusedItems(game, data) {
    game.context.fillStyle = 'black';
    game.context.font = `16px sans-serif`;
    game.context.fillText(`Unused equipment`, 25, 95);

    displayAvailableEquipments(
      game,
      images,
      { x: 25, y: 100, width: 750, height: 350 },
      data.cleanup_equipment_controller,
      {
        can_action: (_equipment) => !data.cleanup_equipment.active,
        on_action: (equipment) => {
          data.unused_equipment = data.unused_equipment.filter((t) => t !== equipment);
          data.cleanup_equipment = { active: true, target: equipment };
          recomputeEquipmentTargetCycle(game.game_data, data);
        },
      },
    );
  }

  /**
   * @param {import("../../engine.mjs").Game} game The current game
   * @param {import('../../types').TownGameData} data The current game
   */
  function displayEquipmentCleanup(game, data) {
    const scrap_button = renderButton(game, !game.game_data.pause.is_paused, {
      x: 185,
      y: 490,
      background: data.cleanup_equipment.active ? { base: '#AAA', hover: '#CCC' } : { base: '#C77', hover: '#C77' },
      content: { type: 'text', text: 'Scrap Item', font_size: 20, padding: { x: 5, y: 5 } },
    });

    if (!data.cleanup_equipment.active) {
      game.context.fillStyle = '#C77';
      display_equipment.detailed_slot.empty(game, { x: 50, y: 470 }, 'No item', undefined);
      if (scrap_button.is_hovered) showPopupOnMousePosition(game, 'Cannot scrap: No item provided');
      if (scrap_button.is_hovered && !game.game_data.pause.is_paused && game.mouse.primary_button_clicked_frame) {
        game.sound_queue.push({ sound: 'error' });
      }
      return;
    }

    display_equipment.detailed_slot.content(
      game,
      images,
      data.cleanup_equipment.target,
      { x: 50, y: 470 },
      {
        text: 'Unassign',
        can_action: () => true,
        action: () => {
          if (!data.cleanup_equipment.active) return;
          data.unused_equipment.push(data.cleanup_equipment.target);
          data.cleanup_equipment = { active: false };
        },
      },
    );
    if (scrap_button.is_hovered) {
      showPopupOnMousePosition(
        game,
        `Destroys the selected item, turning it into ${data.cleanup_equipment.target.level} dust`,
      );
    }
    if (scrap_button.is_hovered && !game.game_data.pause.is_paused && game.mouse.primary_button_clicked_frame) {
      game.sound_queue.push({ sound: 'click' });
      if (data.cleanup_equipment.target.character) {
        game.game_data.game_state.dusted_characters.push(data.cleanup_equipment.target.character.data);
      }
      data.available_dust += data.cleanup_equipment.target.level;
      data.cleanup_equipment = { active: false };
      recomputeEquipmentTargetCycle(game.game_data, data);
    }
  }
}
