import { renderButton } from '../../components/button.mjs';
import {
  displayAvailableEquipments,
  displaySmallAvailableCharacters,
  display_character,
  display_equipment,
} from './utils.mjs';

/**
 * @param {{ [key: string]: ImageBitmap }} images The loaded images to use by the renderer
 */
export function getTownScreenSystem(images) {
  return {
    /**
     * @param {import("../../engine.mjs").Game} game The current game
     * @param {import('../../types').TownGameData} data The current game
     */
    displayTownScreen: function (game, data) {
      displayCharacterRoleChangeInformation(game, data);
      displayEquipmentUpgradeInformation(game, data);
      displayExpeditionInformation(game, data);

      const backstories = renderButton(game, !game.game_data.pause.is_paused, {
        x: 270,
        y: 50,
        background: { base: '#AAA', hover: '#CCC' },
        content: { type: 'fixed_text', text: 'Backstories', font_size: 20, width: 150, height: 40 },
      });
      if (backstories.is_hovered && !game.game_data.pause.is_paused && game.mouse.primary_button_clicked_frame) {
        game.sound_queue.push({ sound: 'click' });
        data.screen = 'backstory';
      }

      const change_role = renderButton(game, !game.game_data.pause.is_paused, {
        x: 50,
        y: 450,
        background: { base: '#AAA', hover: '#CCC' },
        content: { type: 'fixed_text', text: 'Change Role', font_size: 30, width: 200, height: 100 },
      });
      if (change_role.is_hovered && !game.game_data.pause.is_paused && game.mouse.primary_button_clicked_frame) {
        game.sound_queue.push({ sound: 'click' });
        data.screen = 'role_change';
      }
      const equipment_upgrade = renderButton(game, !game.game_data.pause.is_paused, {
        x: 300,
        y: 450,
        background: { base: '#AAA', hover: '#CCC' },
        content: { type: 'fixed_text', text: 'Equipments', font_size: 30, width: 200, height: 80 },
      });
      if (equipment_upgrade.is_hovered && !game.game_data.pause.is_paused && game.mouse.primary_button_clicked_frame) {
        game.sound_queue.push({ sound: 'click' });
        data.screen = 'equipment_upgrade';
      }
      const equipment_cleanup = renderButton(game, !game.game_data.pause.is_paused, {
        x: 300,
        y: 550,
        background: { base: '#AAA', hover: '#CCC' },
        content: { type: 'fixed_text', text: 'Scrap Items', font_size: 25, width: 200, height: 30 },
      });
      if (equipment_cleanup.is_hovered && !game.game_data.pause.is_paused && game.mouse.primary_button_clicked_frame) {
        game.sound_queue.push({ sound: 'click' });
        data.screen = 'equipment_cleanup';
      }
      const expedition = renderButton(game, !game.game_data.pause.is_paused, {
        x: 550,
        y: 450,
        background: { base: '#AAA', hover: '#CCC' },
        content: { type: 'fixed_text', text: 'Expedition', font_size: 30, width: 200, height: 100 },
      });
      if (expedition.is_hovered && !game.game_data.pause.is_paused && game.mouse.primary_button_clicked_frame) {
        game.sound_queue.push({ sound: 'click' });
        data.screen = 'expedition';
      }
    },
  };

  /**
   * @param {import("../../engine.mjs").Game} game The current game
   * @param {import('../../types').TownGameData} data The current game
   */
  function displayCharacterRoleChangeInformation(game, data) {
    displaySmallAvailableCharacters(
      game,
      images,
      { x: 45, y: 265, width: 210, height: 180 },
      data.town_characters_controller,
    );

    if (data.change_role.active) {
      game.context.strokeStyle = 'white';
      game.context.strokeRect(45, 190, 210, 70);
      display_character.simplified_slot.content(game, images, data.change_role.target, { x: 65, y: 200 });
      game.context.fillStyle = '#777';
      game.context.fillRect(120, 200, 130, 50);
      game.context.fillStyle = 'black';
      game.context.font = '15px sans-serif';
      game.context.fillText(`New Role: ${data.change_role.target_role}`, 130, 220, 110);
      const remaining_cycles = data.change_role.retarget_end_cycle - game.game_data.game_state.current_cycle;
      const remaining_text = `Remaining: ${remaining_cycles} ${remaining_cycles > 1 ? 'cycles' : 'cycle'}`;
      game.context.fillText(remaining_text, 130, 240, 110);
    }
  }

  /**
   * @param {import("../../engine.mjs").Game} game The current game
   * @param {import('../../types').TownGameData} data The current game
   */
  function displayEquipmentUpgradeInformation(game, data) {
    displayAvailableEquipments(
      game,
      images,
      { x: 295, y: 265, width: 210, height: 180 },
      data.town_equipment_controller,
    );

    if (data.upgrade_equipment.active) {
      game.context.strokeStyle = 'white';
      game.context.strokeRect(295, 180, 210, 80);
      if (data.upgrade_equipment.handler) {
        display_character.simplified_slot.content(game, images, data.upgrade_equipment.handler, { x: 305, y: 195 });
      } else {
        display_character.simplified_slot.empty(game, { x: 305, y: 195 });
      }
      if (data.upgrade_equipment.target) {
        display_equipment.simplified_slot.content(game, images, data.upgrade_equipment.target, { x: 360, y: 220 });
      } else {
        display_equipment.simplified_slot.empty(game, { x: 360, y: 220 });
      }
      if (data.upgrade_equipment.retarget_end_cycle > 0) {
        game.context.fillStyle = '#7C7';
        game.context.fillRect(360, 190, 135, 25);
        game.context.fillStyle = 'black';
        game.context.font = '15px sans-serif';
        const remaining_cycles = data.upgrade_equipment.retarget_end_cycle - game.game_data.game_state.current_cycle;
        const remaining_text = `Remaining: ${remaining_cycles} ${remaining_cycles > 1 ? 'cycles' : 'cycle'}`;
        game.context.fillText(remaining_text, 365, 210, 125);
      } else {
        game.context.fillStyle = '#C77';
        game.context.fillRect(360, 190, 135, 25);
        game.context.fillStyle = 'black';
        game.context.font = '15px sans-serif';
        game.context.fillText('No valid upgrade', 365, 210, 125);
      }
    }
  }

  /**
   * @param {import("../../engine.mjs").Game} game The current game
   * @param {import('../../types').TownGameData} data The current game
   */
  function displayExpeditionInformation(game, data) {
    game.context.strokeStyle = 'white';
    game.context.strokeRect(545, 215, 210, 230);
    let start_x = 555;
    let start_y = 220;
    for (const slot of data.expedition_slots) {
      if (slot.character) {
        display_character.simplified_slot.content(game, images, slot.character, { x: start_x, y: start_y });
      } else {
        display_character.simplified_slot.empty(game, { x: start_x, y: start_y + 22 });
      }
      if (slot.equipment.body) {
        display_equipment.simplified_slot.content(game, images, slot.equipment.body, {
          x: start_x + 55,
          y: start_y,
        });
      } else {
        display_equipment.simplified_slot.empty(game, { x: start_x + 55, y: start_y });
      }
      if (slot.equipment.accessory) {
        display_equipment.simplified_slot.content(game, images, slot.equipment.accessory, {
          x: start_x + 55,
          y: start_y + 35,
        });
      } else {
        display_equipment.simplified_slot.empty(game, { x: start_x + 55, y: start_y + 35 });
      }
      if (slot.equipment.weapon) {
        display_equipment.simplified_slot.content(game, images, slot.equipment.weapon, {
          x: start_x + 55,
          y: start_y + 70,
        });
      } else {
        display_equipment.simplified_slot.empty(game, { x: start_x + 55, y: start_y + 70 });
      }
      start_x += 100;
      if (start_x + 80 > 750) {
        start_x = 555;
        start_y += 120;
      }
    }
  }
}
